//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2007-2010 by PGMed@FIT, BUT    \n
 *
 * Authors: Ondrej Siler, siler@fit.vutbr.cz    \n
 * Date:    yyyy/mm/dd                          \n
 *
 * $Id: mdsColorTable.h 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Color LUT.
 */

#ifndef _COLOR_TABLE_H
#define _COLOR_TABLE_H

#include <vector>

// namespace
namespace mds
{
namespace cs
{

//! class holds a vector of colors of type T
template <typename T>
class CColorTable
{
public:
	//! default constructor
	CColorTable() {};
	
	//! constructor sets the default color
	CColorTable(T defaultColor) : _defaultColor(defaultColor) {};
	
	//! destructor
	~CColorTable() {};
	
	//! sets the default color
	void SetDefaultColor (T defaultColor) {_defaultColor = defaultColor;}
	
	//! encapsulates the push back operation of std::vector
	void push_back(const T& value) {_colors.push_back(value);}
	
	//! encapsulates operator [] of std::vector, checks bounds. If the index is invalid, returns a default color
	T& operator[](const int index) {return (index<0 || index >= (int)_colors.size()) ? _defaultColor : _colors[index];}

    //! encapsulates clear operation of std::vector
    void clear() {_colors.clear();};

protected:
    std::vector<T> _colors;
	T	_defaultColor;
};


} // namespace
} // namespace

#endif
