//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2007-2010 by PGMed@FIT, BUT    \n
 *
 * Authors: Ondrej Siler, siler@fit.vutbr.cz    \n
 * Date:    yyyy/mm/dd                          \n
 *
 * $Id: mdsTriContainerVCTL.h 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Class for storing triangles created by cross section of tetrahedral mesh.
 */

#ifndef TRI_CONTAINER_VCTL_H
#define TRI_CONTAINER_VCTL_H

#include <MDSTk/CrossSection/mdsTriContainer.h>

#include <VectorEntity/mcvertices.h>
#include <VectorEntity/mctris.h>

// namespace
namespace mds
{
namespace cs
{

//! container of triangles as VectorEntity MCTriS list
class TriContainerVCTL : public CTriContainer, public vctl::MCTriS
{
public:	
	//! constructor
	TriContainerVCTL(bool testExistence = false) {vertices = GetVerticeS(); vertices->SetTestExistence(testExistence);};
	
	//! destructor
	~TriContainerVCTL() {ClearTriValuePtrs();};
	
	//! creates new triangle in the list
	virtual void pushTri(vctl::MCVertex * v1, vctl::MCVertex * v2, vctl::MCVertex * v3, vctl::MCTetra * tetra)
	{
		New(vertices->New(v1), vertices->New(v2), vertices->New(v3))->SetValuePtr(tetra);
	};
	
	//! clears the list
	virtual void clearTris() {ClearTriValuePtrs(); ClearAll(); vertices->ClearAll();};
	
	
	
protected:

	/*! \brief A pointer to the set of vertices. The triangular mesh is built on this set*/
    vctl::MCVerticeS* vertices;	
    
	void ClearTriValuePtrs() 
	{
		vctl::MCTri * aktual = (vctl::MCTri *) GetFirst();
     	while (aktual != NULL)
     	{
			aktual->SetValuePtr(NULL);
         	aktual = (vctl::MCTri *) aktual->GetNext();
     	}
	};
};
} // namespace
} // namespace

#endif
