//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2006/09/24                          \n
 *
 * $Id: mdsTetraData.h 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Additional tetrahedron data.
 */

#ifndef MDS_TETRADATA_H
#define MDS_TETRADATA_H

// MDSTk
#include <MDSTk/Base/mdsAssert.h>
#include <MDSTk/Base/mdsTypes.h>
#include <MDSTk/Base/mdsSmallObject.h>

// VCTL
#include <VectorEntity/mcentity.h>
#include <VectorEntity/mcsphere.h>


namespace mds
{
namespace seg
{

//==============================================================================
/*!
 * Structure containing additional tetrahedron information.
 * - Value pointer of every tetrahedron points to this structure.
 */
//struct STetrahedronInfo
struct STetrahedronInfo : public mds::base::CSmallValueObject<>
{
    //! Circumcircle of the triangle.
    vctl::MCSphere Circumsphere;

    //! Pointer to associated user data.
    void *pData;

    //! Default constructor
    STetrahedronInfo() : pData(NULL) {}
};


//==============================================================================
/*
 * Global functions.
 */

//! Returns pointer to tetrahedron circumsphere.
inline vctl::MCSphere *getTetrahedronCircumsphere(vctl::MCTetra *pTetrahedron)
{
    return &(((STetrahedronInfo *)pTetrahedron->GetValuePtr())->Circumsphere);
}


//! Returns pointer to the user data of a given tetrahedron.
//! - Null if there are no associated data.
inline void *getTetrahedronDataPtr(vctl::MCTetra *pTetrahedron)
{
    return ((STetrahedronInfo *)pTetrahedron->GetValuePtr())->pData;
}


//! Sets pointer to the user data of a given tetrahedron.
inline void setTetrahedronDataPtr(vctl::MCTetra *pTetrahedron, void *pData)
{
    ((STetrahedronInfo *)pTetrahedron->GetValuePtr())->pData = pData;
}


} // namespace seg
} // namespace mds

#endif // MDS_TETRADATA_H

