//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2006/06/12                          \n
 *
 * $Id: mdsTetraMembershipEM.h 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Membership function based on Expection-Maximization (EM) algorithm.
 */

#ifndef MDS_TETRAMEMBERSHIPEM_H
#define MDS_TETRAMEMBERSHIPEM_H

#include "mdsTetraMembershipFunc.h"
#include "mdsTetraContainer.h"

// MDSTk
#include <MDSTk/Math/Algorithm/mdsEM.h>


namespace mds
{
namespace seg
{

//==============================================================================
/*!
 * Region membership function.
 * - Expectation-Maximization (EM) algorithm.
 */
class CTetraMembershipEM : public CTetraMembershipFunc
{
public:
    //! Default constructor.
    CTetraMembershipEM(double dMinAddChange);

    //! Virtual destructor.
    virtual ~CTetraMembershipEM() {}

    //! Computes membership function of all tetrahedra in a given list.
    //! - Return false on failure.
    virtual bool operator()(vctl::MCTetra *pFirstTetrahedron);

    //! Computes membership function of all tetrahedra in a given list.
    //! - Predefined number of clusters.
    //! - Return false on failure.
    virtual bool operator()(vctl::MCTetra *pFirstTetrahedron,
                            mds::tSize NumOfClasses
                            );

protected:
    //! Used clustering algorithm.
    mds::math::CMaxLikelihoodByEM<CTetraContainer, NUM_OF_FEATURES> m_EM;
};


} // namespace seg
} // namespace mds

#endif // MDS_TETRAMEMBERSHIPEM_H

