//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2006/06/12                          \n
 *
 * $Id: mdsTetraMembershipFCM.h 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Membership function based on Fuzzy C-means clustering algortihm.
 */

#ifndef MDS_TETRAMEMBERSHIPFCM_H
#define MDS_TETRAMEMBERSHIPFCM_H

#include "mdsTetraMembershipFunc.h"
#include "mdsTetraContainer.h"

// MDSTk
#include <MDSTk/Math/Algorithm/mdsFuzzyCMeans.h>


namespace mds
{
namespace seg
{

//==============================================================================
/*!
 * Region membership function.
 * - Fuzzy C-means algorithm.
 */
class CTetraMembershipFCM : public CTetraMembershipFunc
{
public:
    CTetraMembershipFCM(double dWeight);

    //! Virtual destructor.
    virtual ~CTetraMembershipFCM() {}

    //! Computes membership function of all tetrahedra in a given list.
    //! - Return false on failure.
    virtual bool operator()(vctl::MCTetra *pFirstTetrahedron);

    //! Computes membership function of all tetrahedra in a given list.
    //! - Predefined number of clusters.
    //! - Return false on failure.
    virtual bool operator()(vctl::MCTetra *pFirstTetrahedron,
                            mds::tSize NumOfClasses
                            );

protected:
    //! Used clustering algorithm.
    mds::math::CFuzzyCMeans<CTetraContainer, NUM_OF_FEATURES> m_FCM;
};


} // namespace seg
} // namespace mds

#endif // MDS_TETRAMEMBERSHIPFCM_H

