//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2006/06/12                          \n
 *
 * $Id: mdsTetraMembershipFunc.h 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Segment membership function.
 */

#ifndef MDS_TETRAMEMBERSHIPFUNC_H
#define MDS_TETRAMEMBERSHIPFUNC_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>

// VCTL
#include <VectorEntity/mcentity.h>

#include "mdsTetraFeatures.h"


namespace mds
{
namespace seg
{

//==============================================================================
/*
 * Global definitions.
 */

//! If this macro is defined, various informations are logged.
#ifndef MDS_SEG_LOGGING_ENABLED
#    define MDS_SEG_LOGGING_ENABLED
#endif


//==============================================================================
/*!
 * Abstract class of a segment membership function.
 */
class CTetraMembershipFunc : public CTetraBase
{
public:
    //! Feature vector.
    typedef CTetraFeatures tFeatures;

    //! Type of a single tetrahedron feature.
    typedef CTetraFeatures::tFeature tFeature;

    //! Number of used triangle features.
    enum { NUM_OF_FEATURES = tFeatures::NUM_OF_FEATURES };

public:
    //! Default constructor.
    CTetraMembershipFunc() : m_NumOfSegments(0) {}

    //! Virtual destructor.
    virtual ~CTetraMembershipFunc() {}

    //! Computes membership function of all tetrahedra in a given list.
    //! - Returns false on failure.
    virtual bool operator()(vctl::MCTetra *pFirstTetrahedron) = 0;

    //! Computes membership function of all tetrahedra in a given list.
    //! - Predefined number of clusters.
    //! - Returns false on failure.
    virtual bool operator()(vctl::MCTetra *pFirstTetrahedron,
                            tSize NumOfClasses
                            ) = 0;

    //! Returns the number of segments
    mds::tSize getNumOfSegments() { return m_NumOfSegments; }

protected:
    //! The number of segments.
    mds::tSize m_NumOfSegments;
};


} // namespace seg
} // namespace mds

#endif // MDS_TETRAMEMBERSHIPFUNC_H

