//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2006/05/06                          \n
 *
 * $Id: mdsTetraVertexDetector.hxx 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Initial vertex detector.
 */

//#define LOG_VERTICES


//==============================================================================
/*
 * Methods templates.
 */

//template <>
bool CTetraVertexDetector<CELL_MAX>::operator()(tVolume *pVolume,
                                                vctl::MCVerticeS *pVertices,
                                                tScale *pScale
                                                )
{
    MDS_ASSERT(pVolume && pVertices && pScale);

    MDS_LOG_NOTE("CTetraVertexDetector::operator()():");

    // Get image size
    mds::tSize XSize = pVolume->getXSize();
    mds::tSize YSize = pVolume->getYSize();
    mds::tSize ZSize = pVolume->getZSize();

    // Estimate threshold
//    double dThreshold = mds::img::getMean<double>(*pVolume) + m_dThreshold * sqrt(mds::img::getVariance<double>(*pVolume));
    double dThreshold = 0.0;

    // Minimal and maximal values
    double dMin = mds::img::getMin<double>(*pVolume);
    double dMax = mds::img::getMax<double>(*pVolume);
    double dTemp = 1.0 / (dMax - dMin + 0.001);

    // Total number of points
    int iCount = 0;

    // Add all points
/*    vctl::MCPoint3D Point;
    mds::tSize i, j, k;
    for( k = 0; k < ZSize; ++k )
    {
        for( j = 0; j < YSize; ++j )
        {
            for( i = 0; i < XSize; ++i )
            {
                double dValue = pVolume->get(i, j, k);
                if( dValue > dThreshold )
                {
                    pScale->conv2Real(i, j, k, &Point);
                    vctl::MCVertex *pVertex = pVertices->New(Point);
                    pVertex->SetValue((dValue - dMin) * dTemp);
                    ++iCount;
                }
            }
        }
    }*/
    
    // Add all corner
    vctl::MCPoint3D Point;
    mds::tSize i, j, k;
    for( k = 0; k < ZSize; ++k )
    {
        for( j = 0; j < YSize; ++j )
        {
            for( i = 0; i < XSize; ++i )
            {
                tVoxel Value = pVolume->get(i, j, k);
                if( Value == mds::img::CPixelTraits<tVoxel>::getPixelMax() )
                {
                    pScale->conv2Real(i, j, k, &Point);
                    vctl::MCVertex *pVertex = pVertices->New(Point);
                    pVertex->SetValue((double(Value) - dMin) * dTemp);
                    ++iCount;
                }
            }
        }
    }
    
    // Helper values
    mds::tSize FullSize = m_CellSize + 2 * m_CellMargin;
//    mds::tSize TempSize = m_CellSize + m_CellMargin;

    // Helper string stream used for logging
#ifdef LOG_VERTICES
    std::stringstream ss;
    ss << "points = [";
#endif //LOG_VERTICES

    // Insert all significant edge points to the list of vertices
    // - Process input volume per blocks
//    vctl::MCPoint3D Point;
//    mds::tSize i, j, k;
    for( k = m_CellMargin; (k + m_CellMargin) < (ZSize - m_CellMargin); k += FullSize )
    {
        mds::tSize MinK = k + m_CellMargin;
//        mds::tSize MaxK = mds::math::getMin(k + TempSize, ZSize);
        mds::tSize MaxK = mds::math::getMin(k + FullSize, ZSize);

        for( j = m_CellMargin; (j + m_CellMargin) < (YSize - m_CellMargin); j += FullSize )
        {
            mds::tSize MinJ = j + m_CellMargin;
//            mds::tSize MaxJ = mds::math::getMin(j + TempSize, YSize);
            mds::tSize MaxJ = mds::math::getMin(j + FullSize, YSize);

            for( i = m_CellMargin; (i + m_CellMargin) < (XSize - m_CellMargin); i += FullSize )
            {
                mds::tSize MinI = i + m_CellMargin;
//                mds::tSize MaxI = mds::math::getMin(i + TempSize, XSize);
                mds::tSize MaxI = mds::math::getMin(i + FullSize, XSize);

                // Find maximum in the current block
                mds::tSize MaxX = MinI;
                mds::tSize MaxY = MinJ;
                mds::tSize MaxZ = MinK;
                double dMaxValue = pVolume->get(MaxX, MaxY, MaxZ);

                for( mds::tSize wk = MinK; wk < MaxK; ++wk )
                {
                    for( mds::tSize wj = MinJ; wj < MaxJ; ++wj )
                    {
                        for( mds::tSize wi = MinI; wi < MaxI; ++wi )
                        {
                            double dValue = pVolume->get(wi, wj, wk);
                            if( dValue > dMaxValue )
                            {
                                dMaxValue = dValue;
                                MaxX = wi;
                                MaxY = wj;
                                MaxZ = wk;
                            }
                        }
                    }
                }

                // Create a new vertex
                if( dMaxValue > dThreshold )
                {
                    pScale->conv2Real(MaxX, MaxY, MaxZ, &Point);
                    vctl::MCVertex *pVertex = pVertices->New(Point);
                    pVertex->SetValue((dMaxValue - dMin) * dTemp);
                    ++iCount;
#ifdef LOG_VERTICES
                    ss << MaxX << ',' << MaxY << ',' << MaxZ << "; ";
#endif //LOG_VERTICES
                }
            }
        }
    }

    // Insert all significant edge points on the front and rear volume side
    for( j = m_CellMargin; (j + m_CellMargin) < (YSize - m_CellMargin); j += FullSize )
    {
        mds::tSize MinJ = j + m_CellMargin;
//        mds::tSize MaxJ = mds::math::getMin(j + TempSize, YSize);
        mds::tSize MaxJ = mds::math::getMin(j + FullSize, YSize);

        for( i = m_CellMargin; (i + m_CellMargin) < (XSize - m_CellMargin); i += FullSize )
        {
            mds::tSize MinI = i + m_CellMargin;
//            mds::tSize MaxI = mds::math::getMin(i + TempSize, XSize);
            mds::tSize MaxI = mds::math::getMin(i + FullSize, XSize);

            // Find maximum in the current block
            mds::tSize Max1X = MinI;
            mds::tSize Max1Y = MinJ;
            mds::tSize Max2X = MinI;
            mds::tSize Max2Y = MinJ;
            double dMax1Value = pVolume->get(Max1X, Max1Y, 0);
            double dMax2Value = pVolume->get(Max2X, Max2Y, ZSize - 1);

            for( mds::tSize wj = MinJ; wj < MaxJ; ++wj )
            {
                for( mds::tSize wi = MinI; wi < MaxI; ++wi )
                {
                    double dValue = pVolume->get(wi, wj, 0);
                    if( dValue > dMax1Value )
                    {
                        dMax1Value = dValue;
                        Max1X = wi;
                        Max1Y = wj;
                    }
                    dValue = pVolume->get(wi, wj, ZSize - 1);
                    if( dValue > dMax2Value )
                    {
                        dMax2Value = dValue;
                        Max2X = wi;
                        Max2Y = wj;
                    }
                }
            }

            // Create a new vertex
            if( dMax1Value > dThreshold )
            {
                pScale->conv2Real(Max1X, Max1Y, 0, &Point);
                vctl::MCVertex *pVertex = pVertices->New(Point);
                pVertex->SetValue((dMax1Value - dMin) * dTemp);
                ++iCount;
#ifdef LOG_VERTICES
                ss << Max1X << ',' << Max1Y << ',' << 0 << "; ";
#endif //LOG_VERTICES
            }
            if( dMax2Value > dThreshold )
            {
                pScale->conv2Real(Max2X, Max2Y, ZSize - 1, &Point);
                vctl::MCVertex *pVertex = pVertices->New(Point);
                pVertex->SetValue((dMax2Value - dMin) * dTemp);
                ++iCount;
#ifdef LOG_VERTICES
                ss << Max2X << ',' << Max2Y << ',' << ZSize - 1 << "; ";
#endif //LOG_VERTICES
            }
        }
    }

    for( k = m_CellMargin; (k + m_CellMargin) < (ZSize - m_CellMargin); k += FullSize )
    {
        mds::tSize MinK = k + m_CellMargin;
//        mds::tSize MaxK = mds::math::getMin(k + TempSize, ZSize);
        mds::tSize MaxK = mds::math::getMin(k + FullSize, ZSize);

        for( i = m_CellMargin; (i + m_CellMargin) < (XSize - m_CellMargin); i += FullSize )
        {
            mds::tSize MinI = i + m_CellMargin;
//            mds::tSize MaxI = mds::math::getMin(i + TempSize, XSize);
            mds::tSize MaxI = mds::math::getMin(i + FullSize, XSize);

            // Find maximum in the current block
            mds::tSize Max1X = MinI;
            mds::tSize Max1Z = MinK;
            mds::tSize Max2X = MinI;
            mds::tSize Max2Z = MinK;
            double dMax1Value = pVolume->get(Max1X, 0, Max1Z);
            double dMax2Value = pVolume->get(Max2X, YSize - 1, Max2Z);

            for( mds::tSize wk = MinK; wk < MaxK; ++wk )
            {
                for( mds::tSize wi = MinI; wi < MaxI; ++wi )
                {
                    double dValue = pVolume->get(wi, 0, wk);
                    if( dValue > dMax1Value )
                    {
                        dMax1Value = dValue;
                        Max1X = wi;
                        Max1Z = wk;
                    }
                    dValue = pVolume->get(wi, YSize - 1, wk);
                    if( dValue > dMax2Value )
                    {
                        dMax2Value = dValue;
                        Max2X = wi;
                        Max2Z = wk;
                    }
                }
            }

            // Create a new vertex
            if( dMax1Value > dThreshold )
            {
                pScale->conv2Real(Max1X, 0, Max1Z, &Point);
                vctl::MCVertex *pVertex = pVertices->New(Point);
                pVertex->SetValue((dMax1Value - dMin) * dTemp);
                ++iCount;
#ifdef LOG_VERTICES
                ss << Max1X << ',' << 0 << ',' << Max1Z << "; ";
#endif //LOG_VERTICES
            }
            if( dMax2Value > dThreshold )
            {
                pScale->conv2Real(Max2X, YSize - 1, Max2Z, &Point);
                vctl::MCVertex *pVertex = pVertices->New(Point);
                pVertex->SetValue((dMax2Value - dMin) * dTemp);
                ++iCount;
#ifdef LOG_VERTICES
                ss << Max2X << ',' << YSize - 1 << ',' << Max2Z << "; ";
#endif //LOG_VERTICES
            }
        }
    }

    for( k = m_CellMargin; (k + m_CellMargin) < (ZSize - m_CellMargin); k += FullSize )
    {
        mds::tSize MinK = k + m_CellMargin;
//        mds::tSize MaxK = mds::math::getMin(k + TempSize, ZSize);
        mds::tSize MaxK = mds::math::getMin(k + FullSize, ZSize);

        for( j = m_CellMargin; (j + m_CellMargin) < (YSize - m_CellMargin); j += FullSize )
        {
            mds::tSize MinJ = j + m_CellMargin;
//            mds::tSize MaxJ = mds::math::getMin(j + TempSize, YSize);
            mds::tSize MaxJ = mds::math::getMin(j + FullSize, YSize);

            // Find maximum in the current block
            mds::tSize Max1Y = MinJ;
            mds::tSize Max1Z = MinK;
            mds::tSize Max2Y = MinJ;
            mds::tSize Max2Z = MinK;
            double dMax1Value = pVolume->get(0, Max1Y, Max1Z);
            double dMax2Value = pVolume->get(XSize - 1, Max2Y, Max2Z);

            for( mds::tSize wk = MinK; wk < MaxK; ++wk )
            {
                for( mds::tSize wj = MinJ; wj < MaxJ; ++wj )
                {
                    double dValue = pVolume->get(0, wj, wk);
                    if( dValue > dMax1Value )
                    {
                        dMax1Value = dValue;
                        Max1Y = wj;
                        Max1Z = wk;
                    }
                    dValue = pVolume->get(XSize - 1, wj, wk);
                    if( dValue > dMax2Value )
                    {
                        dMax2Value = dValue;
                        Max2Y = wj;
                        Max2Z = wk;
                    }
                }
            }

            // Create a new vertex
            if( dMax1Value > dThreshold )
            {
                pScale->conv2Real(0, Max1Y, Max1Z, &Point);
                vctl::MCVertex *pVertex = pVertices->New(Point);
                pVertex->SetValue((dMax1Value - dMin) * dTemp);
                ++iCount;
#ifdef LOG_VERTICES
                ss << 0 << ',' << Max1Y << ',' << Max1Z << "; ";
#endif //LOG_VERTICES
            }
            if( dMax2Value > dThreshold )
            {
                pScale->conv2Real(XSize - 1, Max2Y, Max2Z, &Point);
                vctl::MCVertex *pVertex = pVertices->New(Point);
                pVertex->SetValue((dMax2Value - dMin) * dTemp);
                ++iCount;
#ifdef LOG_VERTICES
                ss << XSize - 1 << ',' << Max2Y << ',' << Max2Z << "; ";
#endif //LOG_VERTICES
            }
        }
    }

    // Logging
#ifdef LOG_VERTICES
    ss << "]';";
    MDS_LOG_NOTE(ss.str());
#endif //LOG_VERTICES*/

    MDS_LOG_NOTE("  Number of boundary vertices = " << iCount);

    // O.K.
    return true;
}

