//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2006/09/24                          \n
 *
 * $Id: mdsTriData.h 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Additional triangle data.
 */

#ifndef MDS_TRIDATA_H
#define MDS_TRIDATA_H

// MDSTk
#include <MDSTk/Base/mdsAssert.h>
#include <MDSTk/Base/mdsTypes.h>

// VCTL
#include <VectorEntity/mcentity.h>
#include <VectorEntity/mcsphere.h>


namespace mds
{
namespace seg
{

//==============================================================================
/*!
 * Structure containing additional triangle information.
 * - Value pointer of every triangle points to this structure.
 */
struct STriangleInfo
{
    //! Circumcircle of the triangle.
    vctl::MCSphere Circumsphere;

    //! Pointer to associated user data.
    void *pData;

    //! Default constructor
    STriangleInfo() : pData(NULL) {}
};


//==============================================================================
/*
 * Global functions.
 */

//! Returns pointer to triangle circumcircle.
inline vctl::MCSphere *getTriangleCircumsphere(vctl::MCTri *pTriangle)
{
    return &(((STriangleInfo *)pTriangle->GetValuePtr())->Circumsphere);
}


//! Returns pointer to the user data of a given triangle.
//! - Null if there are no associated data.
inline void *getTriangleDataPtr(vctl::MCTri *pTriangle)
{
    return ((STriangleInfo *)pTriangle->GetValuePtr())->pData;
}


//! Sets pointer to the user data of a given triangle.
inline void setTriangleDataPtr(vctl::MCTri *pTriangle, void *pData)
{
    ((STriangleInfo *)pTriangle->GetValuePtr())->pData = pData;
}


} // namespace seg
} // namespace mds

#endif // MDS_TRIDATA_H

