//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2005/07/26                          \n
 *
 * $Id: mdsTriMembershipEM.h 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Membership function based on Expection-Maximization (EM) algorithm.
 */

#ifndef MDS_TRIMEMBERSHIPEM_H
#define MDS_TRIMEMBERSHIPEM_H

#define EM_LOGGING_ENABLED

#include "mdsTriMembershipFunc.h"
#include "mdsTriContainer.h"

// MDSTk
#include <MDSTk/Math/Algorithm/mdsEM.h>


namespace mds
{
namespace seg
{

//==============================================================================
/*!
 * Region membership function.
 * - Expectation-Maximization (EM) algorithm.
 */
class CTriMembershipEM : public CTriMembershipFunc
{
public:
    //! Default constructor.
    CTriMembershipEM(double dMinAddChange);

    //! Virtual destructor.
    virtual ~CTriMembershipEM() {}

    //! Computes membership function of all triangles in a given list.
    //! - Return false on failure.
    virtual bool operator()(vctl::MCTri *pFirstTriangle);

    //! Computes membership function of all triangles in a given list.
    //! - Predefined number of clusters.
    //! - Return false on failure.
    virtual bool operator()(vctl::MCTri *pFirstTriangle,
                            mds::tSize NumOfClasses
                            );

protected:
    //! Used clustering algorithm.
    mds::math::CMaxLikelihoodByEM<CTriContainer, NUM_OF_FEATURES> m_EM;
};


} // namespace seg
} // namespace mds

#endif // MDS_TRIMEMBERSHIPEM_H

