//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2005/07/26                          \n
 *
 * $Id: mdsTriMembershipFCM.h 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Membership function based on Fuzzy C-means clustering algortihm.
 */

#ifndef MDS_TRIMEMBERSHIPFCM_H
#define MDS_TRIMEMBERSHIPFCM_H

#define FCM_LOGGING_ENABLED

#include "mdsTriMembershipFunc.h"
#include "mdsTriContainer.h"

// MDSTk
#include <MDSTk/Math/Algorithm/mdsFuzzyCMeans.h>


namespace mds
{
namespace seg
{

//==============================================================================
/*!
 * Region membership function.
 * - Fuzzy C-means algorithm.
 */
class CTriMembershipFCM : public CTriMembershipFunc
{
public:
    CTriMembershipFCM(double dWeight);

    //! Virtual destructor.
    virtual ~CTriMembershipFCM() {}

    //! Computes membership function of all triangles in a given list.
    //! - Return false on failure.
    virtual bool operator()(vctl::MCTri *pFirstTriangle);

    //! Computes membership function of all triangles in a given list.
    //! - Predefined number of clusters.
    //! - Return false on failure.
    virtual bool operator()(vctl::MCTri *pFirstTriangle,
                            mds::tSize NumOfClasses
                            );

protected:
    //! Fuzzy C-Means clustering algorithm.
    mds::math::CFuzzyCMeans<CTriContainer, NUM_OF_FEATURES> m_FCM;
};


} // namespace seg
} // namespace mds

#endif // MDS_TRIMEMBERSHIPFCM_H

