//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2005/07/26                          \n
 *
 * $Id: mdsTriMembershipFunc.h 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Segment membership function.
 */

#ifndef MDS_TRIMEMBERSHIPFUNC_H
#define MDS_TRIMEMBERSHIPFUNC_H

#include "mdsTriFeatures.h"

// VCTL
#include <VectorEntity/mcentity.h>


namespace mds
{
namespace seg
{

//==============================================================================
/*
 * Global definitions.
 */

//! If this macro is defined, various informations are logged.
//#ifndef MDS_SEG_LOGGING_ENABLED
//#    define MDS_SEG_LOGGING_ENABLED
//#endif


//==============================================================================
/*!
 * Abstract class of a segment membership function.
 */
class CTriMembershipFunc : public CTriBase
{
public:
    //! Feature vector.
    typedef CTriFeatures tFeatures;

    //! Type of a single triangle feature.
    typedef CTriFeatures::tFeature tFeature;

    //! Number of used triangle features.
    enum { NUM_OF_FEATURES = tFeatures::NUM_OF_FEATURES };

public:
    //! Default constructor.
    CTriMembershipFunc() : m_NumOfSegments(0) {}

    //! Virtual destructor.
    virtual ~CTriMembershipFunc() {}

    //! Computes membership function of all triangles in a given list.
    //! - Returns false on failure.
    virtual bool operator()(vctl::MCTri *pFirstTriangle) = 0;

    //! Computes membership function of all triangles in a given list.
    //! - Predefined number of segments.
    //! - Returns false on failure.
    virtual bool operator()(vctl::MCTri *pFirstTriangle,
                            tSize NumOfClasses
                            ) = 0;

    //! Returns the number of segments after succesfull classification.
    mds::tSize getNumOfSegments() { return m_NumOfSegments; }

protected:
    //! The number of segments.
    mds::tSize m_NumOfSegments;
};


} // namespace seg
} // namespace mds

#endif // MDS_TRIMEMBERSHIPFUNC_H

