//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2005/07/26                          \n
 *
 * $Id: mdsTriVertexDetector.hxx 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Initial vertex detector.
 */

#define LOG_VERTICES


//==============================================================================
/*
 * Methods templates.
 */

//template <>
bool CTriVertexDetector<CELL_MAX>::operator()(const mds::img::CDImage *pImage,
                                              vctl::MCVerticeS *pVertices
                                              )
{
    MDS_ASSERT(pImage && pVertices);

    // Get image size
    mds::tSize XSize = pImage->getXSize();
    mds::tSize YSize = pImage->getYSize();

    // Estimate threshold
    double dThreshold = mds::img::getMean<double>(*pImage) + m_dThreshold * sqrt(mds::img::getVariance<double>(*pImage));

    // Minimal and maximal values
    double dMin = mds::img::getMin<double>(*pImage);
    double dMax = mds::img::getMax<double>(*pImage);
    double dTemp = 1.0 / (dMax - dMin + 0.001);

    // Helper values
    mds::tSize FullSize = m_CellSize + 2 * m_CellMargin;
    mds::tSize TempSize = m_CellSize + m_CellMargin;

    // Helper string stream used for logging
#ifdef LOG_VERTICES
    std::stringstream ss;
    ss << "points = [";
#endif //LOG_VERTICES

    // Insert all significant edge points to the list of vertices
    for( mds::tSize j = 0; (j + m_CellMargin) < YSize; j += FullSize )
    {
        mds::tSize MinJ = j + m_CellMargin;
        mds::tSize MaxJ = mds::math::getMin(j + TempSize, YSize);

        for( mds::tSize i = 0; (i + m_CellMargin) < XSize; i += FullSize )
        {
            mds::tSize MinI = i + m_CellMargin;
            mds::tSize MaxI = mds::math::getMin(i + TempSize, XSize);

            mds::tSize MaxX = MinI;
            mds::tSize MaxY = MinJ;
            double dMaxValue = (double)pImage->get(MaxX, MaxY);

            for( int wj = MinJ; wj < MaxJ; ++wj )
            {
                for( int wi = MinI; wi < MaxI; ++wi )
                {
                    double dValue = (double)pImage->get(wi, wj);
                    if( dValue > dMaxValue )
                    {
                        dMaxValue = dValue;
                        MaxX = wi;
                        MaxY = wj;
                    }
                }
            }
            if( dMaxValue > dThreshold )
            {
                vctl::MCVertex *pVertex = pVertices->New(MaxX, MaxY, 0.0);
                pVertex->SetValue((dMaxValue - dMin) * dTemp);
#ifdef LOG_VERTICES
                ss << MaxX << ',' << MaxY << "; ";
#endif //LOG_VERTICES
            }
        }
    }

    // Logging
#ifdef LOG_VERTICES
    ss << "]';";
    MDS_LOG_NOTE(ss.str());
#endif //LOG_VERTICES

/*    vctl::MCVertex *pVertex = pVertices->New(50.0, 50.0, 0.0);
    pVertex->SetValue(0.9);
 
    pVertex = pVertices->New(100.0, 50.0, 0.0);
    pVertex->SetValue(0.8);
 
    pVertex = pVertices->New(75.0, 100.0, 0.0);
    pVertex->SetValue(0.7);*/

    // O.K.
    return true;
}

