//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2007-2010 by PGMed@FIT, BUT    \n
 *
 * Authors: Ondrej Siler, siler@fit.vutbr.cz    \n
 * Date:    yyyy/mm/dd                          \n
 *
 * $Id: mdsCrossSection.cpp 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Cross section of a plane and tetrahedral mesh.
 */

#include <MDSTk/CrossSection/mdsCrossSection.h>

template<class T> void mySwap(T &a, T &b) {T tmp = a; a = b; b = tmp;}

using namespace mds::cs;
bool CCrossSection::MakeCrossSection(vctl::MCTetra * tetra)
{		
	assert(tetra);
    
    unsigned char intersections = 0;
	unsigned char verticesInPlane = 0;
	unsigned char faces = 0;
	unsigned char mask;
	int intersectionsCount = 0;
	int verticesInPlaneCount = 0;
	   
    // prochazi pres vsechny hrany
    // najde pruseciky a ulozi je do lokalniho seznamu spolecne s indexy hran, ktere protnul
    int edgeIndex = 0;
    for(int i = 0; i < 3; ++i)
        for(int j = i + 1; j < 4; ++j)
        {
        	switch(_plane.Position(tetra->GetVertex(i), tetra->GetVertex(j), _tmpVertex[edgeIndex]))
            {
                // pridat do seznamu vrcholu
                case vctl::MCPlane::INTERSECTS:
                	VertexIndicesToFaceIndices(i, j, mask);
					faces |= mask;
                	intersections |= 0x1 << edgeIndex;
                	intersectionsCount++;
                break;
				case vctl::MCPlane::FIRST_VERTEX_IN_PLANE:
                    VertexIndicesToFaceIndices(i, j, mask);
					faces |= mask;
                    verticesInPlane |= 0x1 << i;
                break;
				case vctl::MCPlane::SECOND_VERTEX_IN_PLANE:
                    VertexIndicesToFaceIndices(i, j, mask);
					faces |= mask;
                    verticesInPlane |= 0x1 << j;
                break;
                case vctl::MCPlane::BOTH_IN_PLANE:
                    VertexIndicesToFaceIndices(i, j, mask);
					faces |= mask;
                    verticesInPlane |= 0x1 << i;
					verticesInPlane |= 0x1 << j;
                break;
                default:
                break;
            }                            
        edgeIndex++;
        }
      
    vctl::MCTetra * aktual; 
    for(int i = 0; i < 4; i++)
    	if((faces & 0x1 << i) && (aktual = tetra->GetNeighbour(i)))
			if(aktual->GetValueFlag() != _flag)
			{
				_tetraQueue.push(aktual);
    	        aktual->SetValueFlag(_flag);
    		}
    		
    for(int i = 0; i < 4; ++i)
    	if(verticesInPlane & (0x01 << i))
    		verticesInPlaneCount++;
	
	if(!intersectionsCount || intersectionsCount + verticesInPlaneCount < 3)
		return false;
	
	// podle poctu pruseciku se rozhodne, jak rovina protla tetrahedron
	// 0 = neprotla
	// 1 = protla jednu hranu, tzn musi dalsi hrana musi lezet v rovine
	// 2 = protla dve hranu, jeden vrchol musi lezet v rovine
	// 3 = rovina protla tri hrany, prida se trojuhelnik + nastavi ukazatel na tetra
	// 4 = rovina protla 4 hrany, vytvori a pridaji se dva trojuhelniky, nastavi se ukazatele na tetra
	
	
	vctl::MCVertex * v1 = NULL;
	vctl::MCVertex * v2 = NULL;
	vctl::MCVertex * v3 = NULL;
	vctl::MCVertex * v4 = NULL;
		
	if(intersectionsCount == 4)
	{
		// rovina protla ctyri steny a ctyri hrany
        // ctyruhelnik se rozdeli na dva trojuhelniky s kratsi spolecnou hranou
        // spolecna hrana trojuhelniku spojuje protejsi vrcholy ctyruhelnika
        // protejsi vrcholy trojuhelnika lezi na hranach tetrahedru, ktere nemaji spolecny vrchol
        
        // ukazatele na vsechny 4 pruseciky
        int intersectionPointer[4];
        int ii = 0;
        for(int i = 0; i < 4; ++i)
        {
    		while(!(intersections & (0x01 << ii++)))
    		;
    		intersectionPointer[i] = ii - 1;
    	}
        
              
        // nalezeni protilehlych pruseciku
        // protilehle jsou hrany 0-5, 1-4, 2-3
        // po prehozeni jsou to dvojice ukazatelu [0]-[1] [2]-[3] 
        if(intersectionPointer[0] + intersectionPointer[2] == 5)
            mySwap(intersectionPointer[2], intersectionPointer[1]);
        else if(intersectionPointer[0] + intersectionPointer[3] == 5)
            mySwap(intersectionPointer[3], intersectionPointer[1]);
        
        // vytvoreni novych vrcholu v trojuhelnikove siti
        v1 = &_tmpVertex[intersectionPointer[0]];
        v2 = &_tmpVertex[intersectionPointer[1]];
        v3 = &_tmpVertex[intersectionPointer[2]];
        v4 = &_tmpVertex[intersectionPointer[3]];
                
        //v1 a v2 jsou pruseciky hran, ktere nejsou soucasti jedne steny, 
        //stejne tak v3 a v4
        
		if(v1->SquareDistance(*v2) > v3->SquareDistance(*v4))
        {
        //vzdalenost mezi vrcholy v1 a v2 je vetsi nez vzdalenost mezi v3 a v4,
		//po vymeneni bude mensi vzdalenost mezi v1 a v2 nez mezi v3 a v4
			mySwap(v1, v3);
			mySwap(v2, v4);
		}
		//v1-v2 budou tvorit spolecnou hranu obou trojuhelniku
        //vznikne trojuhelnik v1, v2, v3 a trojuhelnik v1, v2, v4
        
        //orientace trojuhleniku
        _vec1.Make(v1, v2);
        _vec2.Make(v1, v3);
        _vec3 = _vec1 % _vec2;
        
        if((_plane.GetNormal() * _vec3) > 0)
        {
            _tris->pushTri(v1, v2, v3, tetra);
            _tris->pushTri(v2, v1, v4, tetra);
        }
        else
        {
            _tris->pushTri(v2, v1, v3, tetra);
            _tris->pushTri(v1, v2, v4, tetra);
        }        
        return true;
    }		
	if(intersectionsCount == 3)
    {
    	// rovina protla tri steny a tri hrany
		// ukazatele na vrcholy
       	int intersectionPointer[3];
        int ii = 0;
        for(int i = 0; i < 3; ++i)
        {
    		while(!(intersections & (0x01 << ii++)))
    		;
    		intersectionPointer[i] = ii - 1;
    	}
    	v1 = &_tmpVertex[intersectionPointer[0]];
        v2 = &_tmpVertex[intersectionPointer[1]];
        v3 = &_tmpVertex[intersectionPointer[2]];
        
	}
	else if(intersectionsCount == 2)
	{
		// rovina protla tri steny, protla dve hrany a prochazi jednim vrcholem tetrahedru
		
		// ukazatele na vrcholy
		int intersectionPointer[3];
        int ii = 0;
        for(int i = 0; i < 2; ++i)
        {
    		while(!(intersections & (0x01 << ii++)))
    		;
    		intersectionPointer[i] = ii - 1;
    	}
    	ii = 0;
        while(!(verticesInPlane & (0x01 << ii++)))
    	;
    	intersectionPointer[2] = ii - 1;
    	    	
    	v1 = &_tmpVertex[intersectionPointer[0]];
        v2 = &_tmpVertex[intersectionPointer[1]];
        v3 = tetra->GetVertex(intersectionPointer[2]);
	}	
	else // intersectionsCount == 1
	{
		// rovina protla dve steny, protla jednu hranu a jednou hranou prochazi (= prochazi tedy dvema vrcholy tetrahedru)
		// ukazatele na vrcholy
		int intersectionPointer[3];
        int ii = 0;
        while(!(intersections & (0x01 << ii++)))
    	;
    	intersectionPointer[0] = ii - 1;
    	ii = 0;
    	for(int i = 1; i < 3; ++i)
        {
    		while(!(verticesInPlane & (0x01 << ii++)))
    		;
    		intersectionPointer[i] = ii - 1;
    	}
    	v1 = &_tmpVertex[intersectionPointer[0]];
        v2 = tetra->GetVertex(intersectionPointer[1]);
        v3 = tetra->GetVertex(intersectionPointer[2]);
	}	
	
	//orientace trojuhleniku
	_vec1.Make(v1, v2);
    _vec2.Make(v1, v3);
	_vec3 = _vec1 % _vec2;
    
    if((_plane.GetNormal() * _vec3) > 0)
        _tris->pushTri(v1, v2, v3, tetra);
    else
        _tris->pushTri(v2, v1, v3, tetra);
    
	return true;
}



// prochazi sit pridavanim okolnich tetrahedru do fronty, pokud byl aktualni tetra protnut(nebo dotcen rovinou)
// pamatuje si prvni tetra z minuleho pruchodu, jinak zacina od prvniho tetra v seznamu site
// pokud prvni neni protnut, priblizuje se pruchodem sousedu pres nejblizssi stenu
// otestovanym tetra se nastavuje flag 
void CCrossSection::make()
{      
    //inicializace
    if(!_tris)
        return;    
    _tris->clearTris();
    if(!_tetras)
        return;
    
    _plane.NormalizeVector();

	// rovina neprotne zjednodusenou "bounding sphere"
    if(fabs(_plane.ToPlaneEquation(&_center)) > _radius)
        return;

    // zacatecni tetra pro pruchod site
    vctl::MCTetra * aktual = (_firstFound) ? _firstFound : (vctl::MCTetra *) _tetras->GetFirst();
    if(!aktual)
        return;

	// nastaveni globalniho aktualniho flagu pro sit
    _flag = _tetras->GetActualValueFlag() + 1;
    _tetras->SetActualValueFlag(_flag);

    // nalezeni prvniho tetra, ktery je protnut rovinou
    aktual->SetValueFlag(_flag);
    MakeCrossSection(aktual);
    while(_tetraQueue.empty())
    {        
        aktual = getClosestTetra(aktual);
        if(!aktual)
            return;
        aktual->SetValueFlag(_flag);
        MakeCrossSection(aktual);
    }
    
    // seminkove prochazeni site
    _firstFound = aktual;
       
    while(!_tetraQueue.empty())
    {        
        MakeCrossSection(_tetraQueue.front());
        _tetraQueue.pop();
    }
        
    return;
}


// najde vraci sousedni tetrahedr pres stenu, ktery je nejbliz rovine
vctl::MCTetra * CCrossSection::getClosestTetra(vctl::MCTetra * tetra)
{
    double dist;	// vzdalenost steny od roviny
    double distMin = DOUBLE_MAX; // nejmensi vzdalenost
    int index = -1;    // index steny s nejmensi vzdalenosti
    vctl::MCVertex * triVertices[3]; // 3 vrcholy tvorici stenu tetra
    vctl::MCTetra * neighbour;	// sousedni tetra pres stenu
    for(int i = 0; i < 4; ++i)
    {
        tetra->GetTri(i, triVertices);   // ziskani souseda     
        // vypocet vzdalenosti steny
        dist = fabs(_plane.ToPlaneEquation(triVertices[0]) + _plane.ToPlaneEquation(triVertices[1]) + _plane.ToPlaneEquation(triVertices[2]));
        // zjisteni nejblizsi steny, pokud za ni existuje soused, pres ktery se jeste neslo
        if((dist < distMin) && (neighbour = tetra->GetNeighbour(i)) && (neighbour->GetValueFlag() != _flag))
        {
            index = i;
            distMin = dist;
        }
    }
	// vrati bud nejblizsiho souseda, nebo NULL, pokud neexistuje (v pripade, ze je na okraji tetra site, mozna zacykleni pri pruchodu??)
    return (index > -1) ? tetra->GetNeighbour(index) : NULL;
}

// zjisti stred tetrahedralni site
void CCrossSection::SetCenter()
{
    if(!_tetras)
		return;
    vctl::MCVertex minVertex, maxVertex;
    _tetras->GetVerticeS()->GetRange(minVertex, maxVertex);
    _center.SetX((maxVertex.GetX() + minVertex.GetX())/2.0);
    _center.SetY((maxVertex.GetY() + minVertex.GetY())/2.0);
    _center.SetZ((maxVertex.GetZ() + minVertex.GetZ())/2.0);
    _radius = minVertex.Distance(maxVertex) / 2.0;
}

// vynuluje flag 
void CCrossSection::ResetFlags(unsigned int flag)
{
    if(!_tetras)
    	return;
	vctl::MCTetra * aktual = (vctl::MCTetra *) _tetras->GetFirst();
    while(aktual)
    {
        aktual->SetValueFlag(flag);
        aktual = (vctl::MCTetra *) aktual->GetNext();
    }
    _tetras->SetActualValueFlag(flag);
}

