//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2006/12/21                          \n
 *
 * $Id: mdsTetraHomogeneityMeasure.cpp 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Tetrahedron homogeneity measures.
 */

#include <MDSTk/VectorSegmentation/mdsTetraHomogeneityMeasure.h>

// STL
#include <cmath>


namespace mds
{
namespace seg
{

//==============================================================================
/*
 * Implementation of the class CTetraVarianceHomogeneity.
 */

const double CTetraVarianceHomogeneity::DEFAULT_THRESHOLD     = 0.9;
const double CTetraVarianceHomogeneity::DEFAULT_RHO           = 15.0;


CTetraVarianceHomogeneity::CTetraVarianceHomogeneity(double dThreshold, double dRho)
    : CTetraHomogeneityMeasure(dThreshold)
{
    MDS_ASSERT(dRho > 0.0);
    
    m_dInvRho = -0.5 / (dRho * dRho);
}


double CTetraVarianceHomogeneity::getHomogeneity(const CTetraFeatures& v1)
{
    return std::exp(v1.getVoxelVariance() * m_dInvRho);
//    return std::exp(v1.getVoxelVariance() * v1.getVoxelVariance() * m_dInvRho);
}


} // namespace seg
} // namespace mds

