//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2006/06/12                          \n
 *
 * $Id: mdsTetraMembershipEM.cpp 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Membership function based on Expection-Maximization (EM) algortihm.
 */

#include <MDSTk/VectorSegmentation/mdsTetraMembershipEM.h>

// MDSTk
#include <MDSTk/Base/mdsGlobalLog.h>


namespace mds
{
namespace seg
{

//==============================================================================
/*
 * Implementation of the mds::seg::CTetraMembershipEM class.
 */
CTetraMembershipEM::CTetraMembershipEM(double dMinAddChange)
    : m_EM(dMinAddChange)
{
}


bool CTetraMembershipEM::operator()(vctl::MCTetra *pFirstTetrahedron)
{
    MDS_CHECK(pFirstTetrahedron, return false);

    // Create a container of tetrahedra
    CTetraContainer Container(pFirstTetrahedron);

    // Classify all tetrahedra
    if( !m_EM.execute(Container) )
    {
        return false;
    }

    // Classify all input tetrahedra
    CTetraContainer::tIterator it = Container.getBegin();
    CTetraContainer::tIterator itEnd = Container.getEnd();
    for( tSize Index = 0; it != itEnd; ++it, ++Index )
    {
        // Check the tetrahedron features
        if( it->isError() )
        {
            continue;
        }
        
        // Set region index of the tetrahedron
        setTetrahedronRegion(it.getTetrahedron(), m_EM.getMembership(Index));
    }
    
    // Set the number of segments
    m_NumOfSegments = m_EM.getNumOfComponents();

    // O.K.
    return true;
}


bool CTetraMembershipEM::operator()(vctl::MCTetra *pFirstTetrahedron,
                                    mds::tSize NumOfClasses
                                    )
{
    MDS_CHECK(pFirstTetrahedron && NumOfClasses > 0, return false);

    // Create a container of tetrahedra
    CTetraContainer Container(pFirstTetrahedron);

    // Classify all tetrahedra
    if( !m_EM.execute(Container, NumOfClasses) )
    {
        return false;
    }

    // Classify all input tetrahedra
    CTetraContainer::tIterator it = Container.getBegin();
    CTetraContainer::tIterator itEnd = Container.getEnd();
    for( tSize Index = 0; it != itEnd; ++it, ++Index )
    {
        // Check the Tetrahedron features
        if( it->isError() )
        {
            continue;
        }
        
        // Set region index of the Tetrahedron
        setTetrahedronRegion(it.getTetrahedron(), m_EM.getMembership(Index));
    }

    // Set the number of segments
    m_NumOfSegments = m_EM.getNumOfComponents();

    // O.K.
    return true;
}


} // namespace seg
} // namespace mds

