//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2006/12/14                          \n
 *
 * $Id: mdsTriHomogeneityMeasure.cpp 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Triangle homogeneity measures.
 */

#include <MDSTk/VectorSegmentation/mdsTriHomogeneityMeasure.h>

// STL
#include <cmath>


namespace mds
{
namespace seg
{

//==============================================================================
/*
 * Implementation of the class CTriVarianceHomogeneity.
 */

const double CTriVarianceHomogeneity::DEFAULT_THRESHOLD     = 0.9;
const double CTriVarianceHomogeneity::DEFAULT_RHO           = 15.0;


CTriVarianceHomogeneity::CTriVarianceHomogeneity(double dThreshold, double dRho)
    : CTriHomogeneityMeasure(dThreshold)
{
    MDS_ASSERT(dRho > 0.0);
    
    m_dInvRho = -0.5 / (dRho * dRho);
}


double CTriVarianceHomogeneity::getHomogeneity(const CTriFeatures& v1)
{
    return std::exp(v1.getPixelVariance() * m_dInvRho);
}


} // namespace seg
} // namespace mds

