//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2005/10/24                          \n
 *
 * $Id: mdsTriMembershipEM.cpp 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Membership function based on Expection-Maximization (EM) algorithm.
 */

#include <MDSTk/VectorSegmentation/mdsTriMembershipEM.h>

// MDSTk
#include <MDSTk/Base/mdsGlobalLog.h>


namespace mds
{
namespace seg
{

//==============================================================================
/*
 * Implementation of the CTriMembershipEM class.
 */
CTriMembershipEM::CTriMembershipEM(double dMinAddChange)
    : m_EM(dMinAddChange)
{
}


bool CTriMembershipEM::operator()(vctl::MCTri *pFirstTriangle)
{
    MDS_CHECK(pFirstTriangle, return false);

    // Create a container of triangles
    CTriContainer Container(pFirstTriangle);

    // Classify all triangles
    if( !m_EM.execute(Container) )
    {
        return false;
    }

    // Classify all input triangles
    CTriContainer::tIterator it = Container.getBegin();
    CTriContainer::tIterator itEnd = Container.getEnd();
    for( tSize Index = 0; it != itEnd; ++it, ++Index )
    {
        // Set region index of the triangle
        setTriangleRegion(it.getTriangle(), m_EM.getMembership(Index));
    }

    // Set the number of segments
    m_NumOfSegments = m_EM.getNumOfComponents();

    // O.K.
    return true;
}


bool CTriMembershipEM::operator()(vctl::MCTri *pFirstTriangle,
                                  mds::tSize NumOfClasses
                                  )
{
    MDS_CHECK(pFirstTriangle && NumOfClasses > 0, return false);

    // Create a container of triangles
    CTriContainer Container(pFirstTriangle);

    // Classify all triangles
    if( !m_EM.execute(Container, NumOfClasses) )
    {
        return false;
    }

    // Classify all input triangles
    CTriContainer::tIterator it = Container.getBegin();
    CTriContainer::tIterator itEnd = Container.getEnd();
    for( tSize Index = 0; it != itEnd; ++it, ++Index )
    {
        // Set region index of the triangle
        setTriangleRegion(it.getTriangle(), m_EM.getMembership(Index));
    }

    // Set the number of segments
    m_NumOfSegments = m_EM.getNumOfComponents();

    // O.K.
    return true;
}


} // namespace seg
} // namespace mds

