//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2005/10/24                          \n
 *
 * $Id: mdsTriMembershipFCM.cpp 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Region membership function based on Fuzzy C-means clustering algorithm.
 */

#include <MDSTk/VectorSegmentation/mdsTriMembershipFCM.h>

// MDSTk
#include <MDSTk/Base/mdsGlobalLog.h>


namespace mds
{
namespace seg
{

//==============================================================================
/*
 * Implementation of the mds::seg::CTriMembershipFCM class.
 */
CTriMembershipFCM::CTriMembershipFCM(double dWeight)
    : m_FCM(dWeight)
{
}


bool CTriMembershipFCM::operator()(vctl::MCTri *pFirstTriangle)
{
    MDS_CHECK(pFirstTriangle, return false);

    // Create a container of triangles
    CTriContainer Container(pFirstTriangle);

    // Classify all triangles
    if( !m_FCM.execute(Container) )
    {
        return false;
    }

    // Classify all input triangles
    CTriContainer::tIterator it = Container.getBegin();
    CTriContainer::tIterator itEnd = Container.getEnd();
    for( tSize Index = 0; it != itEnd; ++it, ++Index )
    {
        // Set region index of the triangle
        setTriangleRegion(it.getTriangle(), m_FCM.getMembership(Index));
    }

    // Set the number of segments
    m_NumOfSegments = m_FCM.getNumOfClusters();

    // O.K.
    return true;
}


bool CTriMembershipFCM::operator()(vctl::MCTri *pFirstTriangle,
                                   mds::tSize NumOfClasses
                                   )
{
    MDS_CHECK(pFirstTriangle && NumOfClasses > 0, return false);

    // Create a container of triangles
    CTriContainer Container(pFirstTriangle);

    // Classify all triangles
    if( !m_FCM.execute(Container, NumOfClasses) )
    {
        return false;
    }

    // Classify all input triangles
    CTriContainer::tIterator it = Container.getBegin();
    CTriContainer::tIterator itEnd = Container.getEnd();
    for( tSize Index = 0; it != itEnd; ++it, ++Index )
    {
        // Set region index of the triangle
        setTriangleRegion(it.getTriangle(), m_FCM.getMembership(Index));
    }

    // Set the number of segments
    m_NumOfSegments = m_FCM.getNumOfClusters();

    // O.K.
    return true;
}


} // namespace seg
} // namespace mds

