//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2010 by Michal Spanel     \n
 *
 * Authors: Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2006/11/20                          \n
 *
 * $Id: mdsTriSimilarityMeasure.cpp 1619 2010-02-11 08:35:23Z spanel $
 *
 * Description:
 * - Triangles similarity measures.
 */

#include <MDSTk/VectorSegmentation/mdsTriSimilarityMeasure.h>


namespace mds
{
namespace seg
{

//==============================================================================
/*
 * Implementation of the class CTriMeanValueSimilarity.
 */

const double CTriMeanValueSimilarity::DEFAULT_THRESHOLD     = 0.1;
const double CTriMeanValueSimilarity::DEFAULT_RHO           = 500.0;


CTriMeanValueSimilarity::CTriMeanValueSimilarity(double dThreshold, double dRho)
    : CTriSimilarityMeasure(dThreshold)
{
    MDS_ASSERT(dRho > 0.0);
    
    m_dInvRho = -0.5 / (dRho * dRho);
}


double CTriMeanValueSimilarity::getSimilarity(const CTriFeatures& v1, const CTriFeatures& v2)
{
//    static const int MAX_NUM = 4096 * 4096;
//    int iCount = (v1.getNumOfPixels() < v2.getNumOfPixels()) ? v1.getNumOfPixels() : v2.getNumOfPixels();
    double dCount = (v1.getNumOfPixels() < v2.getNumOfPixels()) ? v1.getNumOfPixels() : v2.getNumOfPixels();
    if( dCount > 0.0 )
    {
        dCount = 1.0 / dCount;
    }
    
    double dTemp = v1.getPixelMean() - v2.getPixelMean();
//    return (MAX_NUM - iCount) * std::exp(m_dInvRho * dTemp * dTemp);
    return dCount * std::exp(m_dInvRho * dTemp * dTemp);
//    return std::exp(m_dInvRho * dTemp * dTemp);
}


} // namespace seg
} // namespace mds

