//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsMakeTetraMesh.h                  \n
 * Section: mMakeTetraMesh                      \n
 * Date:    2006/05/09                          \n
 *
 * $Id: mdsMakeTetraMesh.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Initializes tetrahedral mesh based on input edge volume.
 * - First step of a segmentation based on Adaptive Delaunay triangulation.
 */

#ifndef MDS_MAKETETRAMESH_H
#define MDS_MAKETETRAMESH_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>


//==============================================================================
/*!
 * Sample module providing image segmentation method.
 */
class CMakeTetraMesh : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CMakeTetraMesh);

public:
    //! Default constructor
    CMakeTetraMesh(const std::string& sDescription);

    //! Virtual destructor
    virtual ~CMakeTetraMesh();

protected:
    //! Virtual method called on startup
    virtual bool startup();

    //! Virtual method called by the processing thread
    virtual bool main();

    //! Called on console shutdown
    virtual void shutdown();

    //! Called on writing a usage statement
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Command line arguments.
    double m_dThreshold;
    int m_iCell, m_iNodes;
    double m_dMinEdgeLength;
    bool m_bCDT;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CMakeTetraMesh::tSmartPtr   CMakeTetraMeshPtr;


#endif // MDS_MAKETETRAMESH_H

