//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2007 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsMakeTetraMeshControlSpace.h      \n
 * Section: mMakeTetraMeshControlSpace          \n
 * Date:    2007/03/02                          \n
 *
 * $Id: mdsMakeTetraMeshControlSpace.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Initializes control space used to construct an anisotropic tetrahedral mesh.
 * - Construction of the control space is based on input edge data.
 */

#ifndef MDS_MAKETETRAMESHCONTROLSPACE_H
#define MDS_MAKETETRAMESHCONTROLSPACE_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>


//==============================================================================
/*!
 * Initializes control space used to construct an anisotropic tetrahedral mesh.
 */
class CMakeTetraMeshControlSpace : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CMakeTetraMeshControlSpace);

public:
    //! Default constructor
    CMakeTetraMeshControlSpace(const std::string& sDescription);

    //! Virtual destructor
    virtual ~CMakeTetraMeshControlSpace();

protected:
    //! Virtual method called on startup
    virtual bool startup();

    //! Virtual method called by the processing thread
    virtual bool main();

    //! Called on console shutdown
    virtual void shutdown();

    //! Called on writing a usage statement
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Command line arguments.
    double m_dMinLength, m_dTetra, m_dK;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CMakeTetraMeshControlSpace::tSmartPtr   CMakeTetraMeshControlSpacePtr;


#endif // MDS_MAKETETRAMESHCONTROLSPACE_H

