//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2005 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsMakeTriMesh.h                    \n
 * Section: mMakeTriMesh                        \n
 * Date:    2005/09/27                          \n
 *
 * $Id: mdsMakeTriMesh.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Initializes trianugular mesh based on input edge image.
 * - First step of a segmentation based on Adaptive Delaunay triangulation.
 */

#ifndef MDS_MAKETRIMESH_H
#define MDS_MAKETRIMESH_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>


//==============================================================================
/*!
 * Sample module providing image segmentation method.
 */
class CMakeTriMesh : public mds::mod::CModule
{
public:
    //! Smart pointer type
    //! - Declares type tSmartPtr
    MDS_SHAREDPTR(CMakeTriMesh);

public:
    //! Default constructor
    CMakeTriMesh(const std::string& sDescription);

    //! Virtual destructor
    virtual ~CMakeTriMesh();

protected:
    //! Virtual method called on startup
    virtual bool startup();

    //! Virtual method called by the processing thread
    virtual bool main();

    //! Called on console shutdown
    virtual void shutdown();

    //! Called on writing a usage statement
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Command line arguments.
    double m_dThreshold;
    int m_iCell, m_iNodes;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CMakeTriMesh::tSmartPtr     CMakeTriMeshPtr;


#endif // MDS_MAKETRIMESH_H

