//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsSaveTetraMesh.h                  \n
 * Section: mSaveTetraMesh                      \n
 * Date:    2006/05/09                          \n
 *
 * $Id: mdsSaveTetraMesh.h 1523 2009-11-24 10:11:12Z spanel $
 *
 * File description:
 * - Saves input tetrahedral mesh to the VRML or STL format.
 */

#ifndef MDS_SAVETETRAMESH_H
#define MDS_SAVETETRAMESH_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>


//==============================================================================
/*!
 * Module saves input tetrahedral mesh to the VRML or STL format.
 */
class CSaveTetraMesh : public mds::mod::CModule
{
public:
    //! Smart pointer type
    //! - Declares type tSmartPtr
    MDS_SHAREDPTR(CSaveTetraMesh);

public:
    //! Default constructor
    CSaveTetraMesh(const std::string& sDescription);

    //! Virtual destructor
    virtual ~CSaveTetraMesh();

protected:
    //! Virtual method called on startup
    virtual bool startup();

    //! Virtual method called by the processing thread
    virtual bool main();

    //! Called on console shutdown
    virtual void shutdown();

    //! Called on writing a usage statement
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Format of resulting data.
    enum EDataFormat
    {
        F_VRML,
        F_STL
    };

    //! Resulting data
    EDataFormat m_eDataFormat;

    //! Index of the region/segment.
    int m_iRegion;
    bool m_bBoundary, m_bCut, m_bSlivers;
    double m_dTetQuality;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CSaveTetraMesh::tSmartPtr   CSaveTetraMeshPtr;


#endif // MDS_SAVETETRAMESH_H

