//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2005 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsSaveTriMesh.h                    \n
 * Section: mSaveTriMesh                        \n
 * Date:    2005/09/28                          \n
 *
 * $Id: mdsSaveTriMesh.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * File description:
 * - Saves an input triangular mesh to the VRML or STL format.
 */

#ifndef MDS_SAVETRIMESH_H
#define MDS_SAVETRIMESH_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>


//==============================================================================
/*!
 * Sample module providing image segmentation method.
 */
class CSaveTriMesh : public mds::mod::CModule
{
public:
    //! Smart pointer type
    //! - Declares type tSmartPtr
    MDS_SHAREDPTR(CSaveTriMesh);

public:
    //! Default constructor
    CSaveTriMesh(const std::string& sDescription);

    //! Virtual destructor
    virtual ~CSaveTriMesh();

protected:
    //! Virtual method called on startup
    virtual bool startup();

    //! Virtual method called by the processing thread
    virtual bool main();

    //! Called on console shutdown
    virtual void shutdown();

    //! Called on writing a usage statement
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Format of resulting tri mesh
    enum EDataFormat
    {
        F_VRML,
        F_STL
    };

    //! Resulting data
    EDataFormat m_eDataFormat;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CSaveTriMesh::tSmartPtr     CSaveTriMeshPtr;


#endif // MDS_SAVETRIMESH_H

