//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)     \n
 * Copyright (c) 2003-2008 by Michal Spanel      \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz   \n
 * File:    mdsTetraMeshApproachEdges.h          \n
 * Section: mTetraMeshApproachEdges              \n
 * Date:    2008/02/12                           \n
 *
 * $Id: mdsTetraMeshApproachEdges.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Introduces all image edge points not present in the mesh.
 */

#ifndef MDS_TetraMeshApproachEdges_H
#define MDS_TetraMeshApproachEdges_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Introduces all image edge points not present in the mesh.
 */
class CTetraMeshApproachEdges : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshApproachEdges);

public:
    //! Default constructor.
    CTetraMeshApproachEdges(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshApproachEdges();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spEdgeVolume, m_spControlSpace;

    //! Command line parameters.
    double m_dMinLength;
    bool m_bCheckIsotropicMinEdgeLength;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshApproachEdges::tSmartPtr   CTetraMeshApproachEdgesPtr;


#endif // MDS_TetraMeshApproachEdges_H

