//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2007 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTetraMeshChangeRegion.h          \n
 * Section: mTetraMeshChangeRegion              \n
 * Date:    2007/03/22                          \n
 *
 * $Id: mdsTetraMeshChangeRegion.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Overrides classification of a specified region.
 */

#ifndef MDS_TETRAMESHCHANGEREGION_H
#define MDS_TETRAMESHCHANGEREGION_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsSlice.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module overrides classification of specified regions.
 */
class CTetraMeshChangeRegion : public mds::mod::CModule
{
public:
    //! Smart pointer type
    //! - Declares type tSmartPtr
    MDS_SHAREDPTR(CTetraMeshChangeRegion);

public:
    //! Default constructor
    CTetraMeshChangeRegion(const std::string& sDescription);

    //! Virtual destructor
    virtual ~CTetraMeshChangeRegion();

protected:
    //! Virtual method called on startup
    virtual bool startup();

    //! Virtual method called by the processing thread
    virtual bool main();

    //! Called on console shutdown
    virtual void shutdown();

    //! Called on writing a usage statement
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Parameters.
    int m_iMin, m_iMax, m_iRegion;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshChangeRegion::tSmartPtr   CTetraMeshChangeRegionPtr;


#endif // MDS_TETRAMESHCHANGEREGION_H

