//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2008 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTetraMeshClassifyAdjacent.h      \n
 * Section: mTetraMeshClassifyAdjacent          \n
 * Date:    2008/02/07                          \n
 *
 * $Id: mdsTetraMeshClassifyAdjacent.h 1537 2009-12-14 15:11:52Z spanel $
 *
 * Description:
 * - Region growing segmentation of tetrahedral meshes.
 * - Non-classified tetrahedra are assigned into existing regions.
 */

#ifndef MDS_TetraMeshClassifyAdjacent_H
#define MDS_TetraMeshClassifyAdjacent_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module provides simple region growing algorithm.
 * - Non-classified tetrahedra are assigned into existing regions.
 */
class CTetraMeshClassifyAdjacent : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshClassifyAdjacent);

public:
    //! Default constructor.
    CTetraMeshClassifyAdjacent(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshClassifyAdjacent();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input tetrahedral mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spVolume, m_spEdgeVolume;

    //! Command line parameters.
    double m_dBoundaryThreshold;
    bool m_bClear;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshClassifyAdjacent::tSmartPtr  CTetraMeshClassifyAdjacentPtr;


#endif // MDS_TetraMeshClassifyAdjacent_H

