//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTetraMeshClassifyEM.h            \n
 * Section: mTetraMeshClassifyEM                \n
 * Date:    2006/06/08                          \n
 *
 * $Id: mdsTetraMeshClassifyEM.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Classification of triangles in a mesh.
 */

#ifndef MDS_TETRAMESHCLASSIFYEM_H
#define MDS_TETRAMESHCLASSIFYEM_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module providing classification of tetrahedra in a mesh.
 */
class CTetraMeshClassifyEM : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshClassifyEM);

public:
    //! Default constructor.
    CTetraMeshClassifyEM(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshClassifyEM();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input tetrahedral mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spVolume;

    //! Command line parameters.
    double m_dMinAddChange;
    mds::tSize m_NumOfClusters;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshClassifyEM::tSmartPtr     CTetraMeshClassifyEMPtr;


#endif // MDS_TETRAMESHCLASSIFYEM_H

