//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTetraMeshClassifyFCM.h           \n
 * Section: mTetraMeshClassifyFCM               \n
 * Date:    2006/06/12                          \n
 *
 * $Id: mdsTetraMeshClassifyFCM.h 1537 2009-12-14 15:11:52Z spanel $
 *
 * Description:
 * - Classification of tetrahedra in a mesh.
 */

#ifndef MDS_TETRAMESHCLASSIFYFCM_H
#define MDS_TETRAMESHCLASSIFYFCM_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module providing classification of tetrahedra in a mesh.
 */
class CTetraMeshClassifyFCM : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshClassifyFCM);

public:
    //! Default constructor.
    CTetraMeshClassifyFCM(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshClassifyFCM();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input tetrahedral mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spVolume, m_spEdgeVolume;

    //! Command line parameters.
    double m_dWeight;
    mds::tSize m_NumOfClusters;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshClassifyFCM::tSmartPtr    CTetraMeshClassifyFCMPtr;


#endif // MDS_TETRAMESHCLASSIFYFCM_H

