//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTetraMeshEdgeSplitting.h         \n
 * Section: mTetraMeshEdgeSplitting             \n
 * Date:    2006/06/07                          \n
 *
 * $Id: mdsTetraMeshEdgeSplitting.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Adaptation of tetrahedral mesh to an underlying volume data.
 * - Edge splitting algorithm.
 */

#ifndef MDS_TETRAMESHEDGESPLITTING_H
#define MDS_TETRAMESHEDGESPLITTING_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>
#include <MDSTk/VectorSegmentation/mdsTetraEdgeIterator.h>


//==============================================================================
/*!
 * Module providing edge splitting algorithm for adaptation of tetrahedral
 * mesh to an underlying volume data.
 */
class CTetraMeshEdgeSplitting : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshEdgeSplitting);

public:
    //! Default constructor.
    CTetraMeshEdgeSplitting(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshEdgeSplitting();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input tetrahedral mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spVolume, m_spEdgeVolume;

    //! Command line parameters.
    double m_dMinLength, m_dOrientThreshold;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshEdgeSplitting::tSmartPtr  CTetraMeshEdgeSplittingPtr;


#endif // MDS_TETRAMESHEDGESPLITTING_H

