//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2009 by Michal Spanel          \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2009/10/01                          \n
 *
 * $Id:$
 *
 * Description:
 * - Extracts image features of all tetrahedra.
 */

#ifndef MDS_TetraMeshExtractFeatures_H
#define MDS_TetraMeshExtractFeatures_H

// MDSTk
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module extracts image features of all tetrahedra in the mesh.
 */
class CTetraMeshExtractFeatures : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshExtractFeatures);

public:
    //! Default constructor.
    CTetraMeshExtractFeatures(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshExtractFeatures();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input tetrahedral mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spVolume, m_spEdgeVolume;

    //! Command line parameters.
    bool m_bCheckEdges;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshExtractFeatures::tSmartPtr     CTetraMeshExtractFeaturesPtr;


#endif // MDS_TetraMeshExtractFeatures_H
