//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2007 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTetraMeshFindIsolatedRegions.h   \n
 * Section: mTetraMeshFindIsolatedRegions       \n
 * Date:    2007/03/22                          \n
 *
 * $Id: mdsTetraMeshFindIsolatedRegions.h 1550 2009-12-21 15:40:37Z spanel $
 *
 * Description:
 * - Identifies individual isolated regions of tetrahedra in the classified mesh.
 */

#ifndef MDS_TETRAMESHFINDISOLATEDREGIONS_H
#define MDS_TETRAMESHFINDISOLATEDREGIONS_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsSlice.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module extracts isolated regions from the mesh (overrides classification).
 */
class CTetraMeshFindIsolatedRegions : public mds::mod::CModule
{
public:
    //! Smart pointer type
    //! - Declares type tSmartPtr
    MDS_SHAREDPTR(CTetraMeshFindIsolatedRegions);

public:
    //! Default constructor
    CTetraMeshFindIsolatedRegions(const std::string& sDescription);

    //! Virtual destructor
    virtual ~CTetraMeshFindIsolatedRegions();

protected:
    //! Virtual method called on startup
    virtual bool startup();

    //! Virtual method called by the processing thread
    virtual bool main();

    //! Called on console shutdown
    virtual void shutdown();

    //! Called on writing a usage statement
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Parameters.
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshFindIsolatedRegions::tSmartPtr   CTetraMeshFindIsolatedRegionsPtr;


#endif // MDS_TETRAMESHFINDISOLATEDREGIONS_H

