//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2009 by Michal Spanel          \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * Date:    2009/10/06                          \n
 *
 * $Id:$
 *
 * Description:
 * - Imports classification results to the mesh.
 */

#ifndef MDS_TetraMeshImportClassification_H
#define MDS_TetraMeshImportClassification_H

// MDSTk
#include <MDSTk/Module/mdsModule.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module imports classification results from a simple text file.
 */
class CTetraMeshImportClassification : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshImportClassification);

public:
    //! Default constructor.
    CTetraMeshImportClassification(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshImportClassification();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input tetrahedral mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Command line parameters.
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshImportClassification::tSmartPtr     CTetraMeshImportClassificationPtr;


#endif // MDS_TetraMeshImportClassification_H
