//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTetraMeshInfo.h                  \n
 * Section: mTetraMeshInfo                      \n
 * Date:    2006/06/02                          \n
 *
 * $Id: mdsTetraMeshInfo.h 1550 2009-12-21 15:40:37Z spanel $
 *
 * Description:
 * - Shows information on tetrahedral mesh.
 */

#ifndef MDS_TETRAMESHINFO_H
#define MDS_TETRAMESHINFO_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsSlice.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module shows information about a tetrahedral mesh.
 */
class CTetraMeshInfo : public mds::mod::CModule
{
public:
    //! Smart pointer type
    //! - Declares type tSmartPtr
    MDS_SHAREDPTR(CTetraMeshInfo);

public:
    //! Default constructor
    CTetraMeshInfo(const std::string& sDescription);

    //! Virtual destructor
    virtual ~CTetraMeshInfo();

protected:
    //! Virtual method called on startup
    virtual bool startup();

    //! Virtual method called by the processing thread
    virtual bool main();

    //! Called on console shutdown
    virtual void shutdown();

    //! Called on writing a usage statement
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Parameters.
    bool m_bShowHistogram, m_bShowVolume, m_bDihedralAngles;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshInfo::tSmartPtr   CTetraMeshInfoPtr;


#endif // MDS_TETRAMESHINFO_H

