//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)      \n
 * Copyright (c) 2003-2007 by Michal Spanel       \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz    \n
 * File:    mdsTetraMeshIsotropicEdgeSplitting.h  \n
 * Section: mTetraMeshIsotropicEdgeSplitting      \n
 * Date:    2007/03/03                            \n
 *
 * $Id: mdsTetraMeshIsotropicEdgeSplitting.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Adaptation of tetrahedral mesh to an underlying volume data.
 * - Isotropic edge splitting algorithm.
 */

#ifndef MDS_TETRAMESHISOTROPICEDGESPLITTING_H
#define MDS_TETRAMESHISOTROPICEDGESPLITTING_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module providing anisotropic edge splitting algorithm for adaptation
 * of tetrahedral mesh to an underlying volume data.
 */
class CTetraMeshIsotropicEdgeSplitting : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshIsotropicEdgeSplitting);

public:
    //! Default constructor.
    CTetraMeshIsotropicEdgeSplitting(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshIsotropicEdgeSplitting();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input tetrahedral mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spVolume, m_spEdgeVolume, m_spControlSpace;

    //! Command line parameters.
    double m_dMinLength, m_dThreshold;
    bool m_bBoundary, m_bCDT;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshIsotropicEdgeSplitting::tSmartPtr  CTetraMeshIsotropicEdgeSplittingPtr;


#endif // MDS_TETRAMESHISOTROPICEDGESPLITTING_H

