//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTetraMeshNoiseReduction.h        \n
 * Section: mTetraMeshNoiseReduction            \n
 * Date:    2006/06/12                          \n
 *
 * $Id: mdsTetraMeshNoiseReduction.cpp 1494 2009-05-21 14:36:00Z spanel $
 *
 * Description:
 * - Re-classification of noisy tetrahedra.
 */

#include "mdsTetraMeshNoiseReduction.h"

// STL
#include <climits>
#include <iostream>
#include <string>


//==============================================================================
/*
 * Global constants.
 */

//! Module description
const std::string MODULE_DESCRIPTION    = "Re-classification of noisy tetrahedra.";

//! Additional command line arguments
const std::string MODULE_ARGUMENTS      = "";

//! Additional arguments names
//const std::string MODULE_ARG_         = "";

//! Default argument values
//const double DEFAULT_ARG_             = ;


//==============================================================================
/*
 * Implementation of the class CTetraMeshNoiseReduction.
 */
CTetraMeshNoiseReduction::CTetraMeshNoiseReduction(const std::string& sDescription)
    : mds::mod::CModule(sDescription)
{
    allowArguments(MODULE_ARGUMENTS);
}


CTetraMeshNoiseReduction::~CTetraMeshNoiseReduction()
{
}


bool CTetraMeshNoiseReduction::startup()
{
    // Note
    MDS_LOG_NOTE("Module startup");

    // Test of existence of input and output channel
//    if( getNumOfInputs() != 2 || getNumOfOutputs() != 1 )
    if( getNumOfInputs() != 1 || getNumOfOutputs() != 1 )
    {
        MDS_CERR('<' << m_sFilename << "> Wrong number of input and output channels" << std::endl);
        return false;
    }

    // Command line parameters
/*    m_d = DEFAULT_ARG_;
    m_Arguments.value(MODULE_ARG_, m_d);
    if( m_d < 1.0 )
    {
        MDS_CERR('<' << m_sFilename << "> Bad " << MODULE_ARG_ << " parameter value: type -h for help" << std::endl);
        printUsage();
        return false;
    }*/

    // O.K.
    return true;
}


bool CTetraMeshNoiseReduction::main()
{
    // Note
    MDS_LOG_NOTE("Module main function");

    // I/O channels
    mds::mod::CChannel *pIChannel = getInput(0);
    mds::mod::CChannel *pOChannel = getOutput(0);

    // Is any input?
    if( !pIChannel->isConnected() )
    {
        return false;
    }

    // Wait for data
    if( !pIChannel->wait(1000) )
    {
        MDS_LOG_NOTE("Wait timeout");
        return true;
    }

    // Read volume from the input channel
/*    if( !readInput(pIChannel, m_spVolume.get()) )
    {
        MDS_CERR('<' << m_sFilename << "> Failed to read input volume" << std::endl);
        return false;
    }
 
    // I/O channels
    pIChannel = getInput(1);
 
    // Is any input?
    if( !pIChannel->isConnected() )
    {
        return false;
    }
 
    // Wait for data
    if( !pIChannel->wait(1000) )
    {
        MDS_LOG_NOTE("Wait timeout");
        return true;
    }*/

    // Read mesh from the input channel
    if( !readInput(pIChannel, m_spMesh.get()) )
    {
        MDS_CERR('<' << m_sFilename << "> Failed to read input tetrahedral mesh" << std::endl);
        return false;
    }

    // Set the underlying volume
//    m_spMesh->setVolume(m_spDensityVolume);

    // Removes (re-classificates) noisy tetrahedra
    m_spMesh->noiseReduction();
    m_spMesh->reassignRegions();

    // Write the result to the output channel
    if( !writeOutput(pOChannel, m_spMesh.get()) )
    {
        MDS_CERR('<' << m_sFilename << "> Failed to write the output tetrahedral mesh" << std::endl);
    }

    // Returning 'true' means to continue processing the input channel
    return false;
}


void CTetraMeshNoiseReduction::shutdown()
{
    // Note
    MDS_LOG_NOTE("Module shutdown");
}


void CTetraMeshNoiseReduction::writeExtendedUsage(std::ostream& Stream)
{
    MDS_CERR(std::endl);
    MDS_CERR("Extended usage:" << std::endl);
    MDS_CERR("Options:" << std::endl);
    MDS_CERR("Input: tetrahedral mesh." << std::endl);
    MDS_CERR("Output: tetrahedral mesh." << std::endl);
}


//==============================================================================
/*
 * Function main() which creates and executes console application.
 */
int main(int argc, char *argv[])
{
    // Creation of a module using smart pointer
    CTetraMeshNoiseReductionPtr spModule(new CTetraMeshNoiseReduction(MODULE_DESCRIPTION));

    // Initialize and execute the module
    if( spModule->init(argc, argv) )
    {
        spModule->run();
    }

    // Console application finished
    return 0;
}

