//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTetraMeshNoiseReduction.h        \n
 * Section: mTetraMeshNoiseReduction            \n
 * Date:    2006/06/12                          \n
 *
 * $Id: mdsTetraMeshNoiseReduction.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Re-classification of noisy tetrahedra.
 */

#ifndef MDS_TETRAMESHNOISEREDUCTION_H
#define MDS_TETRAMESHNOISEREDUCTION_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module providing re-classification of noisy tetrahedra.
 */
class CTetraMeshNoiseReduction : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshNoiseReduction);

public:
    //! Default constructor.
    CTetraMeshNoiseReduction(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshNoiseReduction();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spVolume;

    //! Command line parameters.
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshNoiseReduction::tSmartPtr     CTetraMeshNoiseReductionPtr;


#endif // MDS_TETRAMESHNOISEREDUCTION_H

