//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTetraMeshOptimize.h              \n
 * Section: mTetraMeshOptimize                  \n
 * Date:    2006/06/02                          \n
 *
 * $Id: mdsTetraMeshOptimize.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Tetrahedral mesh optimization.
 */

#ifndef MDS_TETRAMESHOPTIMIZE_H
#define MDS_TETRAMESHOPTIMIZE_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module providing optimization of input tetrahedral mesh.
 */
class CTetraMeshOptimize : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshOptimize);

public:
    //! Default constructor.
    CTetraMeshOptimize(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshOptimize();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spEdgeVolume;

    //! Command line parameters.
    double m_dQuality, m_dEdge;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshOptimize::tSmartPtr   CTetraMeshOptimizePtr;


#endif // MDS_TETRAMESHOPTIMIZE_H

