//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTetraMeshRasterization.h         \n
 * Section: mTetraMeshRasterization             \n
 * Date:    2006/06/12                          \n
 *
 * $Id: mdsTetraMeshRasterization.h 1550 2009-12-21 15:40:37Z spanel $
 *
 * Description:
 * - Rasterization of tetrahedra in a mesh.
 */

#ifndef MDS_TETRAMESHRASTERIZATION_H
#define MDS_TETRAMESHRASTERIZATION_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Math/mdsRandom.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module providing rasterization of a classified tetrahedral mesh.
 */
class CTetraMeshRasterization : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshRasterization);

public:
    //! Default constructor.
    CTetraMeshRasterization(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshRasterization();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input tetrahedral mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spVolume;

    //! Command line parameters.
    int m_iRegion;
    bool m_bRandom, m_bEdges, m_bTriangles, m_bInterpolate;
    double m_dAlpha;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshRasterization::tSmartPtr  CTetraMeshRasterizationPtr;


#endif // MDS_TETRAMESHRASTERIZATION_H

