//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTetraMeshRegionClassifyFCM.h     \n
 * Section: mTetraMeshRegionClassifyFCM         \n
 * Date:    2006/12/28                          \n
 *
 * $Id: mdsTetraMeshRegionClassifyFCM.h 1537 2009-12-14 15:11:52Z spanel $
 *
 * Description:
 * - Classification of sub-regions in a mesh.
 */

#ifndef MDS_TETRAMESHREGIONCLASSIFYFCM_H
#define MDS_TETRAMESHREGIONCLASSIFYFCM_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module providing classification of sub-regions in a mesh.
 */
class CTetraMeshRegionClassifyFCM : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshRegionClassifyFCM);

public:
    //! Default constructor.
    CTetraMeshRegionClassifyFCM(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshRegionClassifyFCM();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input tetrahedral mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spVolume, m_spEdgeVolume;

    //! Command line parameters.
    double m_dWeight;
    mds::tSize m_NumOfClusters;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshRegionClassifyFCM::tSmartPtr  CTetraMeshRegionClassifyFCMPtr;


#endif // MDS_TETRAMESHREGIONCLASSIFYFCM_H

