//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTetraMeshRegionGrowing.h         \n
 * Section: mTetraMeshRegionGrowing             \n
 * Date:    2006/12/28                          \n
 *
 * $Id: mdsTetraMeshRegionGrowing.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Region growing segmentation of tetrahedral meshes.
 */

#ifndef MDS_TETRAMESHREGIONGROWING_H
#define MDS_TETRAMESHREGIONGROWING_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module provides region growing segmentation algorithm.
 */
class CTetraMeshRegionGrowing : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshRegionGrowing);

public:
    //! Default constructor.
    CTetraMeshRegionGrowing(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshRegionGrowing();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input tetrahedral mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spVolume, m_spEdgeVolume;

    //! Command line parameters.
    double m_dBoundaryThreshold, m_dSimilarityThreshold, m_dRho;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshRegionGrowing::tSmartPtr  CTetraMeshRegionGrowingPtr;


#endif // MDS_TETRAMESHREGIONGROWING_H

