//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTetraMeshRegionMerging.h         \n
 * Section: mTetraMeshRegionMerging             \n
 * Date:    2006/12/28                          \n
 *
 * $Id: mdsTetraMeshRegionMerging.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Region merging algorithm.
 */

#ifndef MDS_TETRAMESHREGIONMERGING_H
#define MDS_TETRAMESHREGIONMERGING_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module merges regions until a given similarity predicate is true.
 */
class CTetraMeshRegionMerging : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshRegionMerging);

public:
    //! Default constructor.
    CTetraMeshRegionMerging(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshRegionMerging();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input tetrahedral mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spVolume, m_spEdgeVolume;

    //! Command line parameters.
    double m_dBoundaryThreshold, m_dSimilarityThreshold, m_dRho, m_dDelta;
    int m_iNumOfRegions;
    bool m_bMergeAdjacent;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshRegionMerging::tSmartPtr  CTetraMeshRegionMergingPtr;


#endif // MDS_TETRAMESHREGIONMERGING_H

