//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)     \n
 * Copyright (c) 2003-2007 by Michal Spanel      \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz   \n
 * File:    mdsTetraMeshSubpixelVertexMoving.h   \n
 * Section: mTetraMeshSubpixelVertexMoving       \n
 * Date:    2007/12/05                           \n
 *
 * $Id: mdsTetraMeshSubpixelVertexMoving.h 1550 2009-12-21 15:40:37Z spanel $
 *
 * Description:
 * - Tetrahedral mesh optimization.
 */

#ifndef MDS_TetraMeshSubpixelVertexMoving_H
#define MDS_TetraMeshSubpixelVertexMoving_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module providing adaptation of input tetrahedral mesh by means of vertex moving.
 */
class CTetraMeshSubpixelVertexMoving : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshSubpixelVertexMoving);

public:
    //! Default constructor.
    CTetraMeshSubpixelVertexMoving(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshSubpixelVertexMoving();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spEdgeVolume, m_spVolume;

    //! Command line parameters.
    double m_dMinEdgeLength;
    int m_iNeighbourhood;
    bool m_bCDT;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshSubpixelVertexMoving::tSmartPtr   CTetraMeshSubpixelVertexMovingPtr;


#endif // MDS_TetraMeshSubpixelVertexMoving_H

