//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)        \n
 * Copyright (c) 2003-2006 by Michal Spanel         \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz      \n
 * File:    mdsTetraMeshTetrahedronSplitting.h      \n
 * Section: mTetraMeshTetrahedronSplitting          \n
 * Date:    2006/12/19                              \n
 *
 * $Id: mdsTetraMeshTetrahedronSplitting.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Adaptation of tetrahedral mesh to an underlying volume data.
 * - Tetrahedron splitting algorithm.
 */

#ifndef MDS_TETRAMESHTETRAHEDRONSPLITTING_H
#define MDS_TETRAMESHTETRAHEDRONSPLITTING_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module for adaptation of tetrahedral mesh to an underlying volume data.
 * - Non-homogenous tetrahedra splitting algorithm.
 */
class CTetraMeshTetrahedronSplitting : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshTetrahedronSplitting);

public:
    //! Default constructor.
    CTetraMeshTetrahedronSplitting(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshTetrahedronSplitting();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input tetrahedral mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spVolume, m_spEdgeVolume;

    //! Command line parameters.
    double m_dMinLength, m_dHomogeneityThreshold, m_dRho;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshTetrahedronSplitting::tSmartPtr  CTetraMeshTetrahedronSplittingPtr;


#endif // MDS_TETRAMESHTETRAHEDRONSPLITTING_H

