//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)     \n
 * Copyright (c) 2003-2007 by Michal Spanel      \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz   \n
 * File:    mdsTetraMeshVariationalMeshing.h     \n
 * Section: mTetraMeshVariationalMeshing         \n
 * Date:    2007/10/22                           \n
 *
 * $Id: mdsTetraMeshVariationalMeshing.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Variational approach to tetrahedral isotropic meshing.
 */

#ifndef MDS_TETRAMESHVARIATIONALMESHING_H
#define MDS_TETRAMESHVARIATIONALMESHING_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsDensityVolume.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTetraMesh.h>


//==============================================================================
/*!
 * Module providing variational approach to tetrahedral isotropic meshing.
 */
class CTetraMeshVariationalMeshing : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTetraMeshVariationalMeshing);

public:
    //! Default constructor.
    CTetraMeshVariationalMeshing(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTetraMeshVariationalMeshing();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh.
    mds::seg::CTetraMeshPtr m_spMesh;

    //! Input volume.
    mds::img::CDensityVolumePtr m_spVolume, m_spEdgeVolume, m_spControlSpace;

    //! Command line parameters.
    double m_dMinLength;
    bool m_bCheckIsotropicMinEdgeLength, m_bCDT;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTetraMeshVariationalMeshing::tSmartPtr   CTetraMeshVariationalMeshingPtr;


#endif // MDS_TETRAMESHVARIATIONALMESHING_H

