//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2005 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTriMeshAdapt.h                   \n
 * Section: mTriMeshAdapt                       \n
 * Date:    2005/09/28                          \n
 *
 * $Id: mdsTriMeshAdapt.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Adaptation of a triangular mesh to an underlying image.
 * - An empty module at present!
 */

#ifndef MDS_TRIMESHADAPT_H
#define MDS_TRIMESHADAPT_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsSlice.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTriMesh.h>


//==============================================================================
/*!
 * Adaptation of a triangular mesh to an underlying image.
 */
class CTriMeshAdapt : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTriMeshAdapt);

public:
    //! Default constructor.
    CTriMeshAdapt(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTriMeshAdapt();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh.
    mds::seg::CTriMeshPtr m_spMesh;

    //! Input slice.
    mds::img::CSlicePtr m_spSlice;

    //! Command line parameters.
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTriMeshAdapt::tSmartPtr    CTriMeshAdaptPtr;


#endif // MDS_TRIMESHADAPT_H

