//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2005 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTriMeshClassifyFCM.h             \n
 * Section: mTriMeshClassifyFCM                 \n
 * Date:    2005/10/24                          \n
 *
 * $Id: mdsTriMeshClassifyFCM.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Classification of triangles in a mesh.
 */

#ifndef MDS_TRIMESHCLASSIFYFCM_H
#define MDS_TRIMESHCLASSIFYFCM_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsSlice.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTriMesh.h>


//==============================================================================
/*!
 * Module providing classification of triangles in a mesh.
 */
class CTriMeshClassifyFCM : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTriMeshClassifyFCM);

public:
    //! Default constructor.
    CTriMeshClassifyFCM(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTriMeshClassifyFCM();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh.
    mds::seg::CTriMeshPtr m_spMesh;

    //! Input slice.
    mds::img::CSlicePtr m_spSlice;

    //! Command line parameters.
    double m_dWeight;
    mds::tSize m_NumOfClusters;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTriMeshClassifyFCM::tSmartPtr  CTriMeshClassifyFCMPtr;


#endif // MDS_TRIMESHCLASSIFYFCM_H

