//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2005 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTriMeshEdgeSplitting.h           \n
 * Section: mTriMeshEdgeSplitting               \n
 * Date:    2005/12/15                          \n
 *
 * $Id: mdsTriMeshEdgeSplitting.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Adaptation of a triangular mesh to an underlying image.
 * - Edge splitting algorithm.
 */

#ifndef MDS_TRIMESHEDGESPLITTING_H
#define MDS_TRIMESHEDGESPLITTING_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsSlice.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTriMesh.h>
//#include <MDSTk/VectorSegmentation/mdsEdgeIterator.h>


//==============================================================================
/*!
 * Module provides edge splitting algorithm for adaptation of a triangular
 * mesh to an underlying image.
 */
class CTriMeshEdgeSplitting : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTriMeshEdgeSplitting);

public:
    //! Default constructor.
    CTriMeshEdgeSplitting(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTriMeshEdgeSplitting();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh.
    mds::seg::CTriMeshPtr m_spMesh;

    //! Input slice.
    mds::img::CSlicePtr m_spSlice, m_spEdgeSlice;

    //! Command line parameters.
    double m_dMinLength, m_dOrientThreshold;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTriMeshEdgeSplitting::tSmartPtr    CTriMeshEdgeSplittingPtr;


#endif // MDS_TRIMESHEDGESPLITTING_H

