//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2005 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTriMeshInfo.h                    \n
 * Section: mTriMeshInfo                        \n
 * Date:    2005/10/13                          \n
 *
 * $Id: mdsTriMeshInfo.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Shows information on triangular mesh.
 */

#ifndef MDS_TRIMESHINFO_H
#define MDS_TRIMESHINFO_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsSlice.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTriMesh.h>


//==============================================================================
/*!
 * Sample module providing image segmentation method.
 */
class CTriMeshInfo : public mds::mod::CModule
{
public:
    //! Smart pointer type
    //! - Declares type tSmartPtr
    MDS_SHAREDPTR(CTriMeshInfo);

public:
    //! Default constructor
    CTriMeshInfo(const std::string& sDescription);

    //! Virtual destructor
    virtual ~CTriMeshInfo();

protected:
    //! Virtual method called on startup
    virtual bool startup();

    //! Virtual method called by the processing thread
    virtual bool main();

    //! Called on console shutdown
    virtual void shutdown();

    //! Called on writing a usage statement
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh
    mds::seg::CTriMeshPtr m_spMesh;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTriMeshInfo::tSmartPtr     CTriMeshInfoPtr;


#endif // MDS_TRIMESHINFO_H

