//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2005 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTriMeshOptimize.h                \n
 * Section: mTriMeshOptimize                    \n
 * Date:    2005/10/11                          \n
 *
 * $Id: mdsTriMeshOptimize.h 1550 2009-12-21 15:40:37Z spanel $
 *
 * Description:
 * - Triangular mesh optimization.
 */

#ifndef MDS_TRIMESHOPTIMIZE_H
#define MDS_TRIMESHOPTIMIZE_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsSlice.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTriMesh.h>


//==============================================================================
/*!
 * Module optimizes quality of triangles in a mesh.
 */
class CTriMeshOptimize : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTriMeshOptimize);

public:
    //! Default constructor.
    CTriMeshOptimize(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTriMeshOptimize();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh.
    mds::seg::CTriMeshPtr m_spMesh;

    //! Input slice.
    mds::img::CSlicePtr m_spSlice;

    //! Command line parameters.
    double m_dQuality, m_dEdge;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTriMeshOptimize::tSmartPtr     CTriMeshOptimizePtr;


#endif // MDS_TRIMESHOPTIMIZE_H

