//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTriMeshRegionClassifyFCM.h       \n
 * Section: mTriMeshRegionClassifyFCM           \n
 * Date:    2006/12/01                          \n
 *
 * $Id: mdsTriMeshRegionClassifyFCM.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Classification of sub-regions in a mesh.
 */

#ifndef MDS_TRIMESHREGIONCLASSIFYFCM_H
#define MDS_TRIMESHREGIONCLASSIFYFCM_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsSlice.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTriMesh.h>


//==============================================================================
/*!
 * Module providing classification of triangles in a mesh.
 */
class CTriMeshRegionClassifyFCM : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTriMeshRegionClassifyFCM);

public:
    //! Default constructor.
    CTriMeshRegionClassifyFCM(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTriMeshRegionClassifyFCM();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh.
    mds::seg::CTriMeshPtr m_spMesh;

    //! Input slice.
    mds::img::CSlicePtr m_spSlice;

    //! Command line parameters.
    double m_dWeight;
    mds::tSize m_NumOfClusters;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTriMeshRegionClassifyFCM::tSmartPtr  CTriMeshRegionClassifyFCMPtr;


#endif // MDS_TRIMESHREGIONCLASSIFYFCM_H

