//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTriMeshRegionGrowing.h           \n
 * Section: mTriMeshRegionGrowing               \n
 * Date:    2006/11/27                          \n
 *
 * $Id: mdsTriMeshRegionGrowing.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Region growing segmentation.
 */

#ifndef MDS_TRIMESHREGIONGROWING_H
#define MDS_TRIMESHREGIONGROWING_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsSlice.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTriMesh.h>


//==============================================================================
/*!
 * Module provides region growing segmentation algorithm.
 */
class CTriMeshRegionGrowing : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTriMeshRegionGrowing);

public:
    //! Default constructor.
    CTriMeshRegionGrowing(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTriMeshRegionGrowing();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh.
    mds::seg::CTriMeshPtr m_spMesh;

    //! Input slice.
    mds::img::CSlicePtr m_spSlice, m_spEdgeSlice;

    //! Command line parameters.
    double m_dBoundaryThreshold, m_dSimilarityThreshold, m_dRho;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTriMeshRegionGrowing::tSmartPtr    CTriMeshRegionGrowingPtr;


#endif // MDS_TRIMESHREGIONGROWING_H

