//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTriMeshRegionMerging.h           \n
 * Section: mTriMeshRegionMerging               \n
 * Date:    2006/11/20                          \n
 *
 * $Id: mdsTriMeshRegionMerging.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Region merging algorithm.
 */

#ifndef MDS_TRIMESHREGIONMERGING_H
#define MDS_TRIMESHREGIONMERGING_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsSlice.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTriMesh.h>


//==============================================================================
/*!
 * Module merges regions until a given similarity predicate is true.
 */
class CTriMeshRegionMerging : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTriMeshRegionMerging);

public:
    //! Default constructor.
    CTriMeshRegionMerging(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTriMeshRegionMerging();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh.
    mds::seg::CTriMeshPtr m_spMesh;

    //! Input slice.
    mds::img::CSlicePtr m_spSlice, m_spEdgeSlice;

    //! Command line parameters.
    double m_dBoundaryThreshold, m_dSimilarityThreshold, m_dRho;
    int m_iNumOfRegions;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTriMeshRegionMerging::tSmartPtr    CTriMeshRegionMergingPtr;


#endif // MDS_TRIMESHREGIONMERGING_H

