//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2006 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTriMeshTriangleSplitting.h       \n
 * Section: mTriMeshTriangleSplitting           \n
 * Date:    2006/12/14                          \n
 *
 * $Id: mdsTriMeshTriangleSplitting.h 1446 2009-04-20 13:41:24Z spanel $
 *
 * Description:
 * - Adaptation of a triangular mesh to an underlying image.
 * - Triangle splitting.
 */

#ifndef MDS_TRIMESHTRIANGLESPLITTING_H
#define MDS_TRIMESHTRIANGLESPLITTING_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Module/mdsModule.h>
#include <MDSTk/System/mdsEvent.h>
#include <MDSTk/Image/mdsSlice.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTriMesh.h>


//==============================================================================
/*!
 * Module provides triangle splitting algorithm for adaptation of a triangular
 * mesh to an underlying image.
 */
class CTriMeshTriangleSplitting : public mds::mod::CModule
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTriMeshTriangleSplitting);

public:
    //! Default constructor.
    CTriMeshTriangleSplitting(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTriMeshTriangleSplitting();

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Input triangular mesh.
    mds::seg::CTriMeshPtr m_spMesh;

    //! Input slice.
    mds::img::CSlicePtr m_spSlice, m_spEdgeSlice;

    //! Command line parameters.
    double m_dMinLength, m_dHomogeneityThreshold, m_dRho;
};


//==============================================================================
/*!
 * Smart pointer to console application.
 */
typedef CTriMeshTriangleSplitting::tSmartPtr    CTriMeshTriangleSplittingPtr;


#endif // MDS_TRIMESHTRIANGLESPLITTING_H

