//==============================================================================
/*! \file
 * Medical Data Segmentation Toolkit (MDSTk)    \n
 * Copyright (c) 2003-2005 by Michal Spanel     \n
 *
 * Author:  Michal Spanel, spanel@fit.vutbr.cz  \n
 * File:    mdsTriMeshView.h                    \n
 * Section: mTriMeshView                        \n
 * Date:    2005/08/30                          \n
 *
 * $Id: mdsTriMeshView.h 1550 2009-12-21 15:40:37Z spanel $
 *
 * Description:
 * - Reads a triangular mesh and underlying slice from the input channel.
 * - Draws both using GLUT and OpenGL.
 */

#ifndef MDS_TRIMESHVIEW_H
#define MDS_TRIMESHVIEW_H

// MDSTk
#include <MDSTk/Base/mdsSetup.h>
#include <MDSTk/Base/mdsTypes.h>
#include <MDSTk/Module/mdsView.h>
#include <MDSTk/Image/mdsSlice.h>

// MDSTk Extensions
#include <MDSTk/VectorSegmentation/mdsTriMesh.h>


//==============================================================================
/*!
 * Module visualizes an input mesh (and image) using GLUT and OpenGL.
 */
class CTriMeshView : public mds::mod::CView
{
public:
    //! Smart pointer type.
    //! - Declares type tSmartPtr.
    MDS_SHAREDPTR(CTriMeshView);

public:
    //! Default constructor.
    CTriMeshView(const std::string& sDescription);

    //! Virtual destructor.
    virtual ~CTriMeshView();

    //! Returns pointer to the current view.
    static CTriMeshView *getViewPtr()
    {
        MDS_ASSERT(m_pView);
        return m_pView;
    }

    //! Returns pointer to the input mesh.
    mds::seg::CTriMesh *getMesh() { return m_spMesh; }

    //! Returns pointer to the input slice.
    mds::img::CSlice *getSlice() { return m_spSlice; }

protected:
    //! Virtual method called on startup.
    virtual bool startup();

    //! Virtual method called by the processing thread.
    virtual bool main();

    //! Called on console shutdown.
    virtual void shutdown();

    //! Called on writing a usage statement.
    virtual void writeExtendedUsage(std::ostream& Stream);

protected:
    //! Static pointer to the created view.
    static CTriMeshView *m_pView;

    //! Input triangular mesh.
    mds::seg::CTriMeshPtr m_spMesh;

    //! Input slice.
    mds::img::CSlicePtr m_spSlice;
};


//==============================================================================
/*!
 * Smart pointer to view application.
 */
typedef CTriMeshView::tSmartPtr     CTriMeshViewPtr;


#endif // MDS_TRIMESHVIEW_H

